@@ -1,22 +1,22 @@
-# -*- coding: utf-8 -*-
-from __future__ import absolute_import, division, print_function, unicode_literals
-
 __license__ = "GNU Affero General Public License http://www.gnu.org/licenses/agpl.html"
-__copyright__ = "Copyright (C) 2015 The OctoPrint Project - Released under terms of the AGPLv3 License"
+__copyright__ = "Copyright (C) 2020 The OctoPrint Project - Released under terms of the AGPLv3 License"
 
 
+import datetime
 import logging
 import os
 
 import click
-import zipstream
+from zipstream.ng import ZIP_DEFLATED, ZipStream
 
 from octoprint.cli import init_platform_for_cli, standard_options
 
 click.disable_unicode_literals_warning = True
 
 
-def get_systeminfo(environment_detector, connectivity_checker, additional_fields=None):
+def get_systeminfo(
+    environment_detector, connectivity_checker, settings, additional_fields=None
+):
     from octoprint import __version__
     from octoprint.util import dict_flatten
 
@@ -25,10 +25,30 @@ def get_systeminfo(environment_detector, connectivity_checker, additional_fields
 
     environment_detector.run_detection(notify_plugins=False)
 
+    safe_mode_file = os.path.join(settings.getBaseFolder("data"), "last_safe_mode")
+    last_safe_mode = {"date": "unknown", "reason": "unknown"}
+    try:
+        if os.path.exists(safe_mode_file):
+            with open(safe_mode_file) as f:
+                last_safe_mode["reason"] = f.readline().strip()
+            last_safe_mode["date"] = (
+                datetime.datetime.utcfromtimestamp(
+                    os.path.getmtime(safe_mode_file)
+                ).isoformat()[:19]
+                + "Z"
+            )
+    except Exception as ex:
+        logging.getLogger(__name__).error(
+            "Error while retrieving last safe mode information from {}: {}".format(
+                safe_mode_file, ex
+            )
+        )
+
     systeminfo = {
-        "octoprint": {"version": __version__},
+        "octoprint": {"version": __version__, "last_safe_mode": last_safe_mode},
         "connectivity": connectivity_checker.as_dict(),
         "env": environment_detector.environment,
+        "systeminfo": {"generated": datetime.datetime.utcnow().isoformat()[:19] + "Z"},
     }
 
     # flatten and filter
@@ -45,48 +65,38 @@ def get_systeminfo(environment_detector, connectivity_checker, additional_fields
 def get_systeminfo_bundle(systeminfo, logbase, printer=None, plugin_manager=None):
     from octoprint.util import to_bytes
 
-    systeminfotxt = []
-    for k in sorted(systeminfo.keys()):
-        systeminfotxt.append("{}: {}".format(k, systeminfo[k]))
+    try:
+        z = ZipStream(compress_type=ZIP_DEFLATED)
+    except RuntimeError:
+        # no zlib support
+        z = ZipStream(sized=True)
 
-    terminaltxt = None
     if printer and printer.is_operational():
         firmware_info = printer.firmware_info
         if firmware_info:
+            # add firmware to systeminfo so it's included in systeminfo.txt
             systeminfo["printer.firmware"] = firmware_info["name"]
 
+        # Add printer log, if available
         if hasattr(printer, "_log"):
-            terminaltxt = list(printer._log)
-
-    try:
-        import zlib  # noqa: F401
-
-        compress_type = zipstream.ZIP_DEFLATED
-    except ImportError:
-        # no zlib, no compression
-        compress_type = zipstream.ZIP_STORED
-
-    z = zipstream.ZipFile()
+            z.add(to_bytes("\n".join(printer._log)), arcname="terminal.txt")
 
     # add systeminfo
-    z.writestr(
-        "systeminfo.txt", to_bytes("\n".join(systeminfotxt)), compress_type=compress_type
-    )
+    systeminfotxt = []
+    for k in sorted(systeminfo.keys()):
+        systeminfotxt.append(f"{k}: {systeminfo[k]}")
 
-    # add terminal.txt, if available
-    if terminaltxt:
-        z.writestr(
-            "terminal.txt", to_bytes("\n".join(terminaltxt)), compress_type=compress_type
-        )
+    z.add(to_bytes("\n".join(systeminfotxt)), arcname="systeminfo.txt")
 
     # add logs
     for log in (
         "octoprint.log",
         "serial.log",
+        "tornado.log",
     ):
         logpath = os.path.join(logbase, log)
         if os.path.exists(logpath):
-            z.write(logpath, arcname=log, compress_type=compress_type)
+            z.add_path(logpath, arcname=log)
 
     # add additional bundle contents from bundled plugins
     if plugin_manager:
@@ -106,15 +116,19 @@ def get_systeminfo_bundle(systeminfo, logbase, printer=None, plugin_manager=None
                     if isinstance(content, str):
                         # log path
                         if os.path.exists(content) and os.access(content, os.R_OK):
-                            z.write(content, arcname=log, compress_type=compress_type)
+                            z.add_path(content, arcname=log)
                     elif callable(content):
                         # content generating callable
-                        z.writestr(log, to_bytes(content()), compress_type=compress_type)
+                        try:
+                            z.add(to_bytes(content()), arcname=log)
+                        except Exception:
+                            logging.getLogger(__name__).exception(
+                                f"Error while executing callable for additional bundle content {log} for plugin {name}",
+                                extra={"plugin": name},
+                            )
             except Exception:
                 logging.getLogger(__name__).exception(
-                    "Error while retrieving additional bundle contents for plugin {}".format(
-                        name
-                    ),
+                    f"Error while retrieving additional bundle contents for plugin {name}",
                     extra={"plugin": name},
                 )
 
@@ -128,11 +142,11 @@ def get_systeminfo_bundle_name():
 
 
 @click.group()
-def systeminfo_commands():
+def cli():
     pass
 
 
-@systeminfo_commands.command(name="systeminfo")
+@cli.command(name="systeminfo")
 @standard_options()
 @click.argument(
     "path",
@@ -166,20 +180,17 @@ def systeminfo_command(ctx, path, **kwargs):
         if path:
             # create zip at path
             zipfilename = os.path.join(path, get_systeminfo_bundle_name())
-            click.echo("Writing systeminfo bundle to {}...".format(zipfilename))
+            click.echo(f"Writing systeminfo bundle to {zipfilename}...")
 
             z = get_systeminfo_bundle(
                 systeminfo, settings.getBaseFolder("logs"), plugin_manager=plugin_manager
             )
             try:
                 with open(zipfilename, "wb") as f:
-                    for data in z:
-                        f.write(data)
+                    f.writelines(z)
             except Exception as e:
                 click.echo(str(e), err=True)
-                click.echo(
-                    "There was an error writing to {}.".format(zipfilename), err=True
-                )
+                click.echo(f"There was an error writing to {zipfilename}.", err=True)
                 ctx.exit(-1)
 
             click.echo("Done!")
@@ -188,5 +199,5 @@ def systeminfo_command(ctx, path, **kwargs):
         else:
             # output systeminfo to console
             for k in sorted(systeminfo.keys()):
-                click.echo("{}: {}".format(k, systeminfo[k]))
+                click.echo(f"{k}: {systeminfo[k]}")
     ctx.exit(0)
