@@ -1,21 +1,17 @@
-# -*- coding: utf-8 -*-
-from __future__ import absolute_import, division, print_function, unicode_literals
-
 __author__ = "Gina Häußge <osd@foosel.net>"
 __license__ = "GNU Affero General Public License http://www.gnu.org/licenses/agpl.html"
 __copyright__ = "Copyright (C) 2014 The OctoPrint Project - Released under terms of the AGPLv3 License"
 
-import io
 import logging
 import os
+import time
 from collections import namedtuple
 
-from past.builtins import basestring
-
 import octoprint.plugin
 import octoprint.util
 from octoprint.events import Events, eventManager
 from octoprint.util import get_fully_qualified_classname as fqcn
+from octoprint.util import yaml
 
 from .analysis import AnalysisQueue, QueueEntry  # noqa: F401
 from .destinations import FileDestinations  # noqa: F401
@@ -184,19 +180,19 @@ def get_content_type_mapping_for_extension(extension, subtree=None):
     return None
 
 
-def valid_extension(extension, type=None):
+def valid_extension(extension, type=None, tree=None):
     if not type:
-        return extension in get_all_extensions()
+        return extension in get_all_extensions(subtree=tree)
     else:
-        extensions = get_extensions(type)
+        extensions = get_extensions(type, subtree=tree)
         if extensions:
             return extension in extensions
 
 
-def valid_file_type(filename, type=None):
+def valid_file_type(filename, type=None, tree=None):
     _, extension = os.path.splitext(filename)
     extension = extension[1:].lower()
-    return valid_extension(extension, type=type)
+    return valid_extension(extension, type=type, tree=tree)
 
 
 def get_file_type(filename):
@@ -223,7 +219,7 @@ class NoSuchStorage(Exception):
     pass
 
 
-class FileManager(object):
+class FileManager:
     def __init__(
         self,
         analysis_queue,
@@ -273,9 +269,7 @@ class FileManager(object):
 
         def worker():
             self._logger.info(
-                "Adding backlog items from all storage types to analysis queue...".format(
-                    **locals()
-                )
+                "Adding backlog items from all storage types to analysis queue..."
             )
             for storage_type, storage_manager in self._storage_managers.items():
                 self._determine_analysis_backlog(storage_type, storage_manager)
@@ -332,15 +326,11 @@ class FileManager(object):
 
         if root:
             self._logger.info(
-                'Added {counter} items from storage type "{storage_type}" and root "{root}" to analysis queue'.format(
-                    **locals()
-                )
+                f'Added {counter} items from storage type "{storage_type}" and root "{root}" to analysis queue'
             )
         else:
             self._logger.info(
-                'Added {counter} items from storage type "{storage_type}" to analysis queue'.format(
-                    **locals()
-                )
+                f'Added {counter} items from storage type "{storage_type}" to analysis queue'
             )
 
     def add_storage(self, storage_type, storage_manager):
@@ -456,9 +446,9 @@ class FileManager(object):
                     file_obj = StreamWrapper(
                         os.path.basename(dest_path),
                         io.BytesIO(
-                            ";Generated from {stl_name} (hash: {hash})\n".format(
-                                **locals()
-                            ).encode("ascii", "replace")
+                            f";Generated from {stl_name} (hash: {hash})\n".encode(
+                                "ascii", "replace"
+                            )
                         ),
                         io.FileIO(tmp_path, "rb"),
                     )
@@ -477,7 +467,7 @@ class FileManager(object):
                         analysis=_analysis,
                     )
 
-                    end_time = octoprint.util.monotonic_time()
+                    end_time = time.monotonic()
                     eventManager().fire(
                         Events.SLICING_DONE,
                         {
@@ -508,7 +498,7 @@ class FileManager(object):
 
         slicer = self._slicing_manager.get_slicer(slicer_name)
 
-        start_time = octoprint.util.monotonic_time()
+        start_time = time.monotonic()
         eventManager().fire(
             Events.SLICING_STARTED,
             {
@@ -680,7 +670,7 @@ class FileManager(object):
     ):
         if not destinations:
             destinations = list(self._storage_managers.keys())
-        if isinstance(destinations, basestring):
+        if isinstance(destinations, str):
             destinations = [destinations]
 
         result = {}
@@ -760,6 +750,7 @@ class FileManager(object):
                 "path": path_in_storage,
                 "name": name,
                 "type": get_file_type(name),
+                "operation": "add",
             },
         )
         eventManager().fire(Events.UPDATED_FILES, {"type": "printables"})
@@ -797,6 +788,7 @@ class FileManager(object):
                 "path": path_in_storage,
                 "name": name,
                 "type": get_file_type(name),
+                "operation": "copy",
             },
         )
         eventManager().fire(Events.UPDATED_FILES, {"type": "printables"})
@@ -831,8 +823,22 @@ class FileManager(object):
                 "path": dst_path_in_storage,
                 "name": dst_name,
                 "type": get_file_type(dst_name),
+                "operation": "move",
             },
         )
+        eventManager().fire(
+            Events.FILE_MOVED,
+            {
+                "storage": destination,
+                "source_path": source_path_in_storage,
+                "source_name": source_name,
+                "source_type": get_file_type(source_name),
+                "destination_path": dst_path_in_storage,
+                "destination_name": dst_name,
+                "destination_type": get_file_type(dst_name),
+            },
+        )
+
         eventManager().fire(Events.UPDATED_FILES, {"type": "printables"})
 
     def add_folder(self, destination, path, ignore_existing=True, display=None):
@@ -894,6 +900,17 @@ class FileManager(object):
             Events.FOLDER_ADDED,
             {"storage": destination, "path": dst_path_in_storage, "name": dst_name},
         )
+        eventManager().fire(
+            Events.FOLDER_MOVED,
+            {
+                "storage": destination,
+                "source_path": source_path_in_storage,
+                "source_name": source_name,
+                "destination_path": dst_path_in_storage,
+                "destination_name": dst_name,
+            },
+        )
+
         eventManager().fire(Events.UPDATED_FILES, {"type": "printables"})
 
     def has_analysis(self, destination, path):
@@ -941,8 +958,6 @@ class FileManager(object):
     def save_recovery_data(self, origin, path, pos):
         import time
 
-        import yaml
-
         from octoprint.util import atomic_write
 
         data = {
@@ -953,12 +968,10 @@ class FileManager(object):
         }
         try:
             with atomic_write(self._recovery_file, mode="wt", max_permissions=0o666) as f:
-                yaml.safe_dump(
-                    data, stream=f, default_flow_style=False, indent=2, allow_unicode=True
-                )
+                yaml.save_to_file(data, file=f, pretty=True)
         except Exception:
             self._logger.exception(
-                "Could not write recovery data to file {}".format(self._recovery_file)
+                f"Could not write recovery data to file {self._recovery_file}"
             )
 
     def delete_recovery_data(self):
@@ -969,18 +982,16 @@ class FileManager(object):
             os.remove(self._recovery_file)
         except Exception:
             self._logger.exception(
-                "Error deleting recovery data file {}".format(self._recovery_file)
+                f"Error deleting recovery data file {self._recovery_file}"
             )
 
     def get_recovery_data(self):
         if not os.path.isfile(self._recovery_file):
             return None
 
-        import yaml
-
         try:
-            with io.open(self._recovery_file, "rt", encoding="utf-8") as f:
-                data = yaml.safe_load(f)
+            data = yaml.load_from_file(path=self._recovery_file)
+
             if not isinstance(data, dict) or not all(
                 map(lambda x: x in data, ("origin", "path", "pos", "date"))
             ):
@@ -988,7 +999,7 @@ class FileManager(object):
             return data
         except Exception:
             self._logger.exception(
-                "Could not read recovery data from file {}".format(self._recovery_file)
+                f"Could not read recovery data from file {self._recovery_file}"
             )
             self.delete_recovery_data()
 
@@ -1034,14 +1045,14 @@ class FileManager(object):
 
     def _storage(self, destination):
         if destination not in self._storage_managers:
-            raise NoSuchStorage(
-                "No storage configured for destination {destination}".format(**locals())
-            )
+            raise NoSuchStorage(f"No storage configured for destination {destination}")
         return self._storage_managers[destination]
 
     def _add_analysis_result(self, destination, path, result):
         if destination not in self._storage_managers:
             return
+        if not result:
+            return
 
         storage_manager = self._storage_managers[destination]
         storage_manager.set_additional_metadata(path, "analysis", result, overwrite=True)
