@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 """
 In this module resides the core data structures and logic of the plugin system.
 
@@ -18,7 +17,6 @@ In this module resides the core data structures and logic of the plugin system.
    :members:
 
 """
-from __future__ import absolute_import, division, print_function, unicode_literals
 
 __license__ = "GNU Affero General Public License http://www.gnu.org/licenses/agpl.html"
 __copyright__ = "Copyright (C) 2014 The OctoPrint Project - Released under terms of the AGPLv3 License"
@@ -26,32 +24,18 @@ __copyright__ = "Copyright (C) 2014 The OctoPrint Project - Released under terms
 
 import fnmatch
 import inspect
-import io
 import logging
 import os
-import sys
 from collections import OrderedDict, defaultdict, namedtuple
+from os import scandir
 
 import pkg_resources
 import pkginfo
-from past.builtins import unicode
 
+import octoprint.vendor.imp as imp
 from octoprint.util import sv, time_this, to_unicode
 from octoprint.util.version import get_python_version_string, is_python_compatible
 
-try:
-    from os import scandir
-except ImportError:
-    from scandir import scandir
-
-
-if sys.version_info[0] == 2:
-    # noinspection PyDeprecation
-    import imp
-else:
-    # deprecated in Python 3.4+ and hence vendored for now
-    import octoprint.vendor.imp as imp
-
 
 # noinspection PyDeprecation
 def _find_module(name, path=None):
@@ -85,12 +69,12 @@ def parse_plugin_metadata(path):
         # we only support parsing plain text source files
         return result
 
-    logger.debug("Parsing plugin metadata from AST of {}".format(path))
+    logger.debug(f"Parsing plugin metadata from AST of {path}")
 
     try:
         import ast
 
-        with io.open(path, "rb") as f:
+        with open(path, "rb") as f:
             root = ast.parse(f.read(), filename=path)
 
         assignments = list(
@@ -166,12 +150,12 @@ def parse_plugin_metadata(path):
     except SyntaxError:
         raise
     except Exception:
-        logger.exception("Error while parsing AST from {}".format(path))
+        logger.exception(f"Error while parsing AST from {path}")
 
     return result
 
 
-class ControlProperties(object):
+class ControlProperties:
     attr_name = "__plugin_name__"
     """ Module attribute from which to retrieve the plugin's human readable name. """
 
@@ -193,6 +177,9 @@ class ControlProperties(object):
     attr_license = "__plugin_license__"
     """ Module attribute from which to retrieve the plugin's license. """
 
+    attr_privacypolicy = "__plugin_privacypolicy__"
+    """ Module attribute from which to retrieve the plugin's privacy policy URL, if any. """
+
     attr_pythoncompat = "__plugin_pythoncompat__"
     """
     Module attribute from which to retrieve the plugin's python compatibility string.
@@ -313,7 +300,7 @@ class ModuleOrigin(_ModuleOrigin):
     """
 
 
-class PluginInfo(object):
+class PluginInfo:
     """
     The :class:`PluginInfo` class wraps all available information about a registered plugin.
 
@@ -432,7 +419,7 @@ class PluginInfo(object):
 
     def __str__(self):
         if self.version:
-            return "{name} ({version})".format(name=self.name, version=self.version)
+            return f"{self.name} ({self.version})"
         else:
             return to_unicode(self.name)
 
@@ -485,7 +472,7 @@ class PluginInfo(object):
         else:
             ret = ""
 
-        ret += unicode(self)
+        ret += str(self)
 
         if show_bundled:
             ret += (
@@ -590,7 +577,9 @@ class PluginInfo(object):
             self._version
             if self._version is not None
             else self._get_instance_attribute(
-                ControlProperties.attr_version, default=self._version, incl_metadata=True
+                ControlProperties.attr_version,
+                default=self._version,
+                incl_metadata=True,
             )
         )
 
@@ -633,6 +622,19 @@ class PluginInfo(object):
             ControlProperties.attr_license, default=self._license, incl_metadata=True
         )
 
+    @property
+    def privacypolicy(self):
+        """
+        Privacy Policy URL of the plugin. Will be taken from the privacy policy attribute of the plugin module
+        as defined in :attr:`attr_privacypolicy` if available. May be None.
+
+        Returns:
+            str or None: Privacy Policy URL of the plugin.
+        """
+        return self._get_instance_attribute(
+            ControlProperties.attr_privacypolicy, default=None, incl_metadata=True
+        )
+
     @property
     def pythoncompat(self):
         """
@@ -797,14 +799,12 @@ class PluginInfo(object):
         try:
             return parse_plugin_metadata(self.location)
         except SyntaxError:
-            self._logger.exception(
-                "Invalid syntax in plugin file of plugin {}".format(self.key)
-            )
+            self._logger.exception(f"Invalid syntax in plugin file of plugin {self.key}")
             self.invalid_syntax = True
             return {}
 
 
-class PluginManager(object):
+class PluginManager:
     """
     The :class:`PluginManager` is the central component for finding, loading and accessing plugins provided to the
     system.
@@ -814,6 +814,7 @@ class PluginManager(object):
 
     plugin_timings_logtarget = "PLUGIN_TIMINGS"
     plugin_timings_message = "{func} - {timing:05.2f}ms"
+    default_order = 1000
 
     def __init__(
         self,
@@ -822,6 +823,7 @@ class PluginManager(object):
         plugin_entry_points,
         logging_prefix=None,
         plugin_disabled_list=None,
+        plugin_sorting_order=None,
         plugin_blacklist=None,
         plugin_restart_needing_hooks=None,
         plugin_obsolete_hooks=None,
@@ -841,6 +843,8 @@ class PluginManager(object):
             plugin_entry_points = []
         if plugin_disabled_list is None:
             plugin_disabled_list = []
+        if plugin_sorting_order is None:
+            plugin_sorting_order = {}
         if plugin_blacklist is None:
             plugin_blacklist = []
         if compatibility_ignored_list is None:
@@ -868,6 +872,7 @@ class PluginManager(object):
         self.plugin_bases = plugin_bases
         self.plugin_entry_points = plugin_entry_points
         self.plugin_disabled_list = plugin_disabled_list
+        self.plugin_sorting_order = plugin_sorting_order
         self.plugin_blacklist = processed_blacklist
         self.plugin_restart_needing_hooks = plugin_restart_needing_hooks
         self.plugin_obsolete_hooks = plugin_obsolete_hooks
@@ -907,13 +912,9 @@ class PluginManager(object):
 
     def _detect_python_environment(self):
         import sys
-        from distutils.command.install import install as cmd_install
-        from distutils.dist import Distribution
+        import sysconfig
 
-        cmd = cmd_install(Distribution())
-        cmd.finalize_options()
-
-        self._python_install_dir = cmd.install_lib
+        self._python_install_dir = sysconfig.get_path("purelib")
         self._python_prefix = os.path.realpath(sys.prefix)
         self._python_virtual_env = hasattr(sys, "real_prefix") or (
             hasattr(sys, "base_prefix")
@@ -960,7 +961,9 @@ class PluginManager(object):
         if self.plugin_folders:
             try:
                 added, found = self._find_plugins_from_folders(
-                    self.plugin_folders, existing, ignored_uninstalled=ignore_uninstalled
+                    self.plugin_folders,
+                    existing,
+                    ignored_uninstalled=ignore_uninstalled,
                 )
                 result_added.update(added)
                 result_found += found
@@ -1044,7 +1047,7 @@ class PluginManager(object):
 
                         module_name = None
                         if package:
-                            module_name = "{}.{}".format(package, key)
+                            module_name = f"{package}.{key}"
 
                         plugin = self._import_plugin_from_module(
                             key, module_name=module_name, folder=folder, bundled=bundled
@@ -1068,7 +1071,7 @@ class PluginManager(object):
                             )
                         )
             except Exception:
-                self.logger.exception("Error processing folder {}".format(folder))
+                self.logger.exception(f"Error processing folder {folder}")
 
         return added, found
 
@@ -1206,7 +1209,7 @@ class PluginManager(object):
             else:
                 return None
         except Exception:
-            self.logger.exception("Could not locate plugin {key}".format(key=key))
+            self.logger.exception(f"Could not locate plugin {key}")
             return None
 
         # Create a simple dummy entry first ...
@@ -1224,14 +1227,14 @@ class PluginManager(object):
         plugin.bundled = bundled
 
         if self._is_plugin_disabled(key):
-            self.logger.info("Plugin {} is disabled.".format(plugin))
+            self.logger.info(f"Plugin {plugin} is disabled.")
             plugin.forced_disabled = True
 
         if self._is_plugin_blacklisted(key) or (
             plugin.version is not None
             and self._is_plugin_version_blacklisted(key, plugin.version)
         ):
-            self.logger.warning("Plugin {} is blacklisted.".format(plugin))
+            self.logger.warning(f"Plugin {plugin} is blacklisted.")
             plugin.blacklisted = True
 
         python_version = get_python_version_string()
@@ -1307,7 +1310,7 @@ class PluginManager(object):
 
             plugin.bundled = bundled
         except Exception:
-            self.logger.exception("Error loading plugin {key}".format(key=key))
+            self.logger.exception(f"Error loading plugin {key}")
             return None
 
         if plugin.check():
@@ -1429,7 +1432,7 @@ class PluginManager(object):
                 ):
                     if plugin.blacklisted:
                         self.logger.warning(
-                            "Plugin {} is blacklisted. Not enabling it.".format(plugin)
+                            f"Plugin {plugin} is blacklisted. Not enabling it."
                         )
                         continue
                     self.enable_plugin(
@@ -1469,9 +1472,7 @@ class PluginManager(object):
                 **flags (dict): dictionary of flag names and values
         """
         if name not in self.plugins:
-            self.logger.debug(
-                "Trying to mark an unknown plugin {name}".format(**locals())
-            )
+            self.logger.debug(f"Trying to mark an unknown plugin {name}")
 
         for key, value in flags.items():
             if value is None:
@@ -1502,9 +1503,7 @@ class PluginManager(object):
         self, name, plugin=None, startup=False, initialize_implementation=True
     ):
         if name not in self.plugins:
-            self.logger.warning(
-                "Trying to load an unknown plugin {name}".format(**locals())
-            )
+            self.logger.warning(f"Trying to load an unknown plugin {name}")
             return
 
         if plugin is None:
@@ -1521,7 +1520,7 @@ class PluginManager(object):
             self.on_plugin_loaded(name, plugin)
             plugin.loaded = True
 
-            self.logger.debug("Loaded plugin {name}: {plugin}".format(**locals()))
+            self.logger.debug(f"Loaded plugin {name}: {plugin}")
         except PluginLifecycleException as e:
             raise e
         except Exception:
@@ -1529,9 +1528,7 @@ class PluginManager(object):
 
     def unload_plugin(self, name):
         if name not in self.plugins:
-            self.logger.warning(
-                "Trying to unload unknown plugin {name}".format(**locals())
-            )
+            self.logger.warning(f"Trying to unload unknown plugin {name}")
             return
 
         plugin = self.plugins[name]
@@ -1551,13 +1548,11 @@ class PluginManager(object):
 
             plugin.loaded = False
 
-            self.logger.debug("Unloaded plugin {name}: {plugin}".format(**locals()))
+            self.logger.debug(f"Unloaded plugin {name}: {plugin}")
         except PluginLifecycleException as e:
             raise e
         except Exception:
-            self.logger.exception(
-                "There was an error unloading plugin {name}".format(**locals())
-            )
+            self.logger.exception(f"There was an error unloading plugin {name}")
 
             # make sure the plugin is NOT in the list of enabled plugins but in the list of disabled plugins
             if name in self.enabled_plugins:
@@ -1571,9 +1566,7 @@ class PluginManager(object):
         """Enables a plugin"""
         if name not in self.disabled_plugins:
             self.logger.warning(
-                "Tried to enable plugin {name}, however it is not disabled".format(
-                    **locals()
-                )
+                f"Tried to enable plugin {name}, however it is not disabled"
             )
             return
 
@@ -1600,9 +1593,7 @@ class PluginManager(object):
         except PluginLifecycleException as e:
             raise e
         except Exception:
-            self.logger.exception(
-                "There was an error while enabling plugin {name}".format(**locals())
-            )
+            self.logger.exception(f"There was an error while enabling plugin {name}")
             return False
         else:
             if name in self.disabled_plugins:
@@ -1617,7 +1608,7 @@ class PluginManager(object):
                 plugin.implementation.on_plugin_enabled()
             self.on_plugin_enabled(name, plugin)
 
-            self.logger.debug("Enabled plugin {name}: {plugin}".format(**locals()))
+            self.logger.debug(f"Enabled plugin {name}: {plugin}")
 
         return True
 
@@ -1625,9 +1616,7 @@ class PluginManager(object):
         """Disables a plugin"""
         if name not in self.enabled_plugins:
             self.logger.warning(
-                "Tried to disable plugin {name}, however it is not enabled".format(
-                    **locals()
-                )
+                f"Tried to disable plugin {name}, however it is not enabled"
             )
             return
 
@@ -1643,9 +1632,7 @@ class PluginManager(object):
         except PluginLifecycleException as e:
             raise e
         except Exception:
-            self.logger.exception(
-                "There was an error while disabling plugin {name}".format(**locals())
-            )
+            self.logger.exception(f"There was an error while disabling plugin {name}")
             return False
         else:
             if name in self.enabled_plugins:
@@ -1657,7 +1644,7 @@ class PluginManager(object):
                 plugin.implementation.on_plugin_disabled()
             self.on_plugin_disabled(name, plugin)
 
-            self.logger.debug("Disabled plugin {name}: {plugin}".format(**locals()))
+            self.logger.debug(f"Disabled plugin {name}: {plugin}")
 
         return True
 
@@ -1676,6 +1663,13 @@ class PluginManager(object):
                 )
                 continue
 
+            if not order:
+                order = self.default_order
+
+            override = self.plugin_sorting_order.get(name, {}).get(hook, None)
+            if override:
+                order = override
+
             self._plugin_hooks[hook].append((order, name, callback))
             self._sort_hooks(hook)
 
@@ -1719,6 +1713,9 @@ class PluginManager(object):
                 )
                 continue
 
+            if not order:
+                order = self.default_order
+
             try:
                 self._plugin_hooks[hook].remove((order, name, callback))
                 self._sort_hooks(hook)
@@ -1798,7 +1795,10 @@ class PluginManager(object):
         plugin_hooks = plugin.hooks.keys()
 
         return any(
-            map(lambda hook: PluginManager.hook_matches_hooks(hook, *hooks), plugin_hooks)
+            map(
+                lambda hook: PluginManager.hook_matches_hooks(hook, *hooks),
+                plugin_hooks,
+            )
         )
 
     @staticmethod
@@ -1995,17 +1995,13 @@ class PluginManager(object):
                 raise e
             else:
                 self.logger.exception(
-                    "Exception while initializing plugin {name}, disabling it".format(
-                        **locals()
-                    )
+                    f"Exception while initializing plugin {name}, disabling it"
                 )
                 return False
         else:
             self.on_plugin_implementations_initialized(name, plugin)
 
-        self.logger.debug(
-            "Initialized plugin mixin implementation for plugin {name}".format(**locals())
-        )
+        self.logger.debug(f"Initialized plugin mixin implementation for plugin {name}")
         return True
 
     def log_all_plugins(
@@ -2046,7 +2042,7 @@ class PluginManager(object):
                         show_enabled=show_enabled,
                         enabled_strs=enabled_str,
                     ),
-                    sorted(self.plugins.values(), key=lambda x: unicode(x).lower()),
+                    sorted(self.plugins.values(), key=lambda x: str(x).lower()),
                 )
             )
             legend = "Prefix legend: {1} = disabled, {2} = blacklisted, {3} = incompatible".format(
@@ -2167,12 +2163,19 @@ class PluginManager(object):
                                 impl[0], sorting_context
                             )
                         )
-                        sorting_value = None
+                        sorting_value = self.default_order
+                else:
+                    sorting_value = self.default_order
+
+                override = self.plugin_sorting_order.get(impl[0], {}).get(
+                    sorting_context, None
+                )
+                if override:
+                    sorting_value = override
 
             plugin_info = self.get_plugin_info(impl[0], require_enabled=False)
             return (
-                sorting_value is None,
-                sv(sorting_value),
+                sv(sorting_value, default_value=self.default_order),
                 not plugin_info.bundled if plugin_info else True,
                 sv(impl[0]),
             )
@@ -2265,7 +2268,7 @@ class PluginManager(object):
     def _sort_hooks(self, hook):
         self._plugin_hooks[hook] = sorted(
             self._plugin_hooks[hook],
-            key=lambda x: (x[0] is None, sv(x[0]), sv(x[1]), sv(x[2])),
+            key=lambda x: (sv(x[0]), sv(x[1]), sv(x[2])),
         )
 
     def _get_callback_and_order(self, hook):
@@ -2315,11 +2318,11 @@ def is_sub_path_of(path, parent):
 
 
 def is_editable_install(install_dir, package, module, location):
-    package_link = os.path.join(install_dir, "{}.egg-link".format(package))
+    package_link = os.path.join(install_dir, f"{package}.egg-link")
     if os.path.isfile(package_link):
         expected_target = os.path.normcase(os.path.realpath(location))
         try:
-            with io.open(package_link, "rt", encoding="utf-8") as f:
+            with open(package_link, encoding="utf-8") as f:
                 contents = f.readlines()
             for line in contents:
                 target = os.path.normcase(
@@ -2347,20 +2350,19 @@ class EntryPointMetadata(pkginfo.Distribution):
             for metadata_file in metadata_files:
                 try:
                     return self.entry_point.dist.get_metadata(metadata_file)
-                except (IOError, OSError):  # noqa: B014
+                except OSError:  # noqa: B014
                     # file not found, metadata file might be missing, ignore
-                    # IOError: file not found in Py2
-                    # OSError (specifically FileNotFoundError): file not found in Py3
                     pass
 
         warnings.warn(
             "No package metadata found for package {}".format(
                 self.entry_point.module_name
-            )
+            ),
+            stacklevel=2,
         )
 
 
-class Plugin(object):
+class Plugin:
     """
     The parent class of all plugin implementations.
 
@@ -2401,9 +2403,15 @@ class Plugin(object):
         pass
 
     def on_plugin_enabled(self):
+        """
+        Called by the plugin core when the plugin was enabled. Override this to react to the event.
+        """
         pass
 
     def on_plugin_disabled(self):
+        """
+        Called by the plugin core when the plugin was disabled. Override this to react to the event.
+        """
         pass
 
 
@@ -2444,11 +2452,7 @@ class PluginNeedsRestart(Exception):
     def __init__(self, name):
         Exception.__init__(self)
         self.name = name
-        self.message = (
-            "Plugin {name} cannot be enabled or disabled after system startup".format(
-                **locals()
-            )
-        )
+        self.message = f"Plugin {name} cannot be enabled or disabled after system startup"
 
 
 class PluginLifecycleException(Exception):
