@@ -8,15 +8,17 @@ $(function () {
 
         self.sizeThresholdStr = undefined;
         self.mobileSizeThresholdStr = undefined;
+        self.compressionSizeThresholdStr = undefined;
 
+        self.ui_progress_busy = ko.observable(false);
         self.ui_progress_percentage = ko.observable();
         self.ui_progress_type = ko.observable();
         self.ui_progress_text = ko.pureComputed(function () {
             var text = "";
             switch (self.ui_progress_type()) {
-                case "loading": {
+                case "parsing": {
                     text =
-                        gettext("Loading...") +
+                        gettext("Parsing...") +
                         " (" +
                         self.ui_progress_percentage().toFixed(0) +
                         "%)";
@@ -31,7 +33,7 @@ $(function () {
                     break;
                 }
                 case "done": {
-                    text = gettext("Analyzed");
+                    text = gettext("Ready!");
                     break;
                 }
             }
@@ -59,7 +61,11 @@ $(function () {
         self.renderer_showMoves = ko.observable(true);
         self.renderer_showRetracts = ko.observable(true);
         self.renderer_showPrinthead = ko.observable(true);
+        self.renderer_showSegmentStarts = ko.observable(false);
+        self.renderer_showDebugArcs = ko.observable(false);
+        self.renderer_chromeArcFix = ko.observable(OctoPrint.coreui.browser.chrome);
         self.renderer_showBoundingBox = ko.observable(false);
+        self.renderer_showLayerBoundingBox = ko.observable(false);
         self.renderer_showFullSize = ko.observable(false);
         self.renderer_extrusionWidthEnabled = ko.observable(false);
         self.renderer_extrusionWidth = ko.observable(2);
@@ -68,9 +74,9 @@ $(function () {
         self.renderer_showPrevious = ko.observable(false);
         self.renderer_syncProgress = ko.observable(true);
 
-        self.reader_sortLayers = ko.observable(true);
         self.reader_hideEmptyLayers = ko.observable(true);
         self.reader_ignoreOutsideBed = ko.observable(true);
+        self.reader_forceCompression = ko.observable(false);
 
         self.layerSelectionEnabled = ko.observable(false);
         self.layerUpEnabled = ko.observable(false);
@@ -94,7 +100,11 @@ $(function () {
                 showMoves: self.renderer_showMoves(),
                 showRetracts: self.renderer_showRetracts(),
                 showHead: self.renderer_showPrinthead(),
+                showSegmentStarts: self.renderer_showSegmentStarts(),
+                showDebugArcs: self.renderer_showDebugArcs(),
+                chromeArcFix: self.renderer_chromeArcFix(),
                 showBoundingBox: self.renderer_showBoundingBox(),
+                showLayerBoundingBox: self.renderer_showLayerBoundingBox(),
                 showFullSize: self.renderer_showFullSize(),
                 extrusionWidth: self.renderer_extrusionWidthEnabled()
                     ? self.renderer_extrusionWidth()
@@ -110,9 +120,13 @@ $(function () {
             }
 
             var reader = {
-                sortLayers: self.reader_sortLayers(),
                 purgeEmptyLayers: self.reader_hideEmptyLayers(),
-                ignoreOutsideBed: self.reader_ignoreOutsideBed()
+                ignoreOutsideBed: self.reader_ignoreOutsideBed(),
+                alwaysCompress:
+                    self.settings.settings.plugins.gcodeviewer.alwaysCompress(),
+                compressionSizeThreshold:
+                    self.settings.settings.plugins.gcodeviewer.compressionSizeThreshold(),
+                forceCompression: self.reader_forceCompression()
             };
             if (additionalReaderOptions) {
                 _.extend(reader, additionalReaderOptions);
@@ -141,7 +155,11 @@ $(function () {
         self.renderer_showMoves.subscribe(self.rendererOptionUpdated);
         self.renderer_showRetracts.subscribe(self.rendererOptionUpdated);
         self.renderer_showPrinthead.subscribe(self.rendererOptionUpdated);
+        self.renderer_showSegmentStarts.subscribe(self.rendererOptionUpdated);
+        self.renderer_showDebugArcs.subscribe(self.rendererOptionUpdated);
+        self.renderer_chromeArcFix.subscribe(self.rendererOptionUpdated);
         self.renderer_showBoundingBox.subscribe(self.rendererOptionUpdated);
+        self.renderer_showLayerBoundingBox.subscribe(self.rendererOptionUpdated);
         self.renderer_showFullSize.subscribe(self.rendererOptionUpdated);
         self.renderer_extrusionWidthEnabled.subscribe(self.rendererOptionUpdated);
         self.renderer_extrusionWidth.subscribe(self.rendererOptionUpdated);
@@ -149,9 +167,9 @@ $(function () {
         self.renderer_showCurrent.subscribe(self.rendererOptionUpdated);
         self.renderer_showPrevious.subscribe(self.rendererOptionUpdated);
 
-        self.reader_sortLayers.subscribe(self.readerOptionUpdated);
         self.reader_hideEmptyLayers.subscribe(self.readerOptionUpdated);
         self.reader_ignoreOutsideBed.subscribe(self.readerOptionUpdated);
+        self.reader_forceCompression.subscribe(self.readerOptionUpdated);
 
         self._printerProfileUpdated = function () {
             if (!self.enabled) return;
@@ -208,6 +226,9 @@ $(function () {
                     self.settings.printerProfiles
                         .currentProfileData()
                         .volume.formFactor.subscribe(self._printerProfileUpdated);
+                    self.settings.printerProfiles
+                        .currentProfileData()
+                        .volume.origin.subscribe(self._printerProfileUpdated);
                 }
                 if (self.settings.printerProfiles.currentProfileData().axes) {
                     self.settings.printerProfiles
@@ -318,13 +339,12 @@ $(function () {
 
         self.loadedFilepath = undefined;
         self.loadedFileDate = undefined;
+        self.loadedFileSize = undefined;
         self.status = "idle";
         self.enabled = false;
 
         self.currentlyPrinting = false;
 
-        self.errorCount = 0;
-
         self.layerSlider = undefined;
         self.layerCommandSlider = undefined;
 
@@ -339,6 +359,9 @@ $(function () {
             self.mobileSizeThresholdStr = sizeObservable(
                 self.settings.settings.plugins.gcodeviewer.mobileSizeThreshold
             );
+            self.compressionSizeThresholdStr = sizeObservable(
+                self.settings.settings.plugins.gcodeviewer.compressionSizeThreshold
+            );
 
             var layerSliderElement = $("#gcode_slider_layers");
             var commandSliderElement = $("#gcode_slider_commands");
@@ -356,6 +379,7 @@ $(function () {
 
             self._configureLayerSlider(layerSliderElement);
             self._configureLayerCommandSlider(commandSliderElement);
+            self._configureContainerElement(containerElement);
 
             self.settings.firstRequest.done(function () {
                 var initResult = GCODE.ui.init({
@@ -363,9 +387,11 @@ $(function () {
                     onProgress: self._onProgress,
                     onModelLoaded: self._onModelLoaded,
                     onLayerSelected: self._onLayerSelected,
+                    onFileLoaded: self._onFileLoaded,
                     bed: self._retrieveBedDimensions(),
                     toolOffsets: self._retrieveToolOffsets(),
-                    invertAxes: self._retrieveAxesConfiguration()
+                    invertAxes: self._retrieveAxesConfiguration(),
+                    bedZ: self.settings.settings.gcodeAnalysis.bedZ()
                 });
 
                 if (!initResult) {
@@ -373,6 +399,13 @@ $(function () {
                     return;
                 }
 
+                self.settings.settings.plugins.gcodeviewer.alwaysCompress.subscribe(
+                    self.readerOptionUpdated
+                );
+                self.settings.settings.plugins.gcodeviewer.compressionSizeThreshold.subscribe(
+                    self.readerOptionUpdated
+                );
+
                 self.synchronizeOptions();
                 self.enabled = true;
                 self._fromLocalStorage();
@@ -383,9 +416,14 @@ $(function () {
             self.enableReload(false);
             self.loadedFilepath = undefined;
             self.loadedFileDate = undefined;
+            self.loadedFileSize = undefined;
             self.clear();
         };
 
+        self.resetViewport = function () {
+            GCODE.ui.resetViewport();
+        };
+
         self.resetOptions = function () {
             self.renderer_centerModel(false);
             self.renderer_centerViewport(false);
@@ -393,7 +431,11 @@ $(function () {
             self.renderer_showMoves(true);
             self.renderer_showRetracts(true);
             self.renderer_showPrinthead(true);
+            self.renderer_showSegmentStarts(false);
+            self.renderer_showDebugArcs(false);
+            self.renderer_chromeArcFix(OctoPrint.coreui.browser.chrome);
             self.renderer_showBoundingBox(false);
+            self.renderer_showLayerBoundingBox(false);
             self.renderer_showFullSize(false);
             self.renderer_extrusionWidthEnabled(false);
             self.renderer_extrusionWidth(2);
@@ -402,9 +444,9 @@ $(function () {
             self.renderer_showPrevious(false);
             self.renderer_syncProgress(true);
 
-            self.reader_sortLayers(true);
             self.reader_hideEmptyLayers(true);
             self.reader_ignoreOutsideBed(true);
+            self.reader_forceCompression(false);
         };
 
         self.clear = function () {
@@ -445,55 +487,75 @@ $(function () {
                     max: 1,
                     step: 1,
                     value: [0, 1],
-                    enabled: false,
-                    tooltip: "hide"
+                    enabled: false
+                    //tooltip: "hide"
                 })
                 .on("slide", self.changeCommandRange);
         };
 
-        self.loadFile = function (path, date) {
+        self._configureContainerElement = function (containerElement) {
+            // Prevent the default browser action for the mouse wheel down event. The desired behavior is to have the
+            // gcode canvas pan on mouse down + drag, which happens. But if we don't prevent the default action, the browser
+            // will also scroll the entire page.
+            containerElement.mousedown(function (event) {
+                // Middle mouse button
+                if (event.which === 2) {
+                    event.preventDefault();
+                }
+            });
+        };
+
+        self.loadFile = function (path, date, size, force) {
             self.enableReload(false);
             self.needsLoad = false;
-            if (self.status === "idle" && self.errorCount < 3) {
+            if (
+                self.status === "idle" &&
+                (force ||
+                    (self.cachedPath !== path &&
+                        self.cachedDate !== date &&
+                        self.cachedSize !== size))
+            ) {
                 self.status = "request";
-                OctoPrint.files
-                    .download("local", path)
-                    .done(function (response, rstatus) {
-                        if (rstatus === "success") {
-                            self.showGCodeViewer(response, rstatus);
-                            self.loadedFilepath = path;
-                            self.loadedFileDate = date;
-                            self.status = "idle";
-                            self.enableReload(true);
-                        }
-                    })
-                    .fail(function () {
-                        self.status = "idle";
-                        self.errorCount++;
-                    });
+
+                self.cachedPath = path;
+                self.cachedDate = date;
+                self.cachedSize = size;
+                var par = {
+                    url: OctoPrint.files.downloadPath("local", path),
+                    path: path,
+                    size: size,
+                    skipUntil: self.settings.settings.plugins.gcodeviewer.skipUntilThis()
+                };
+
+                GCODE.renderer.clear();
+                self._onProgress("parsing");
+                GCODE.gCodeReader.loadFile(par);
+
+                if (self.layerSlider !== undefined) {
+                    self.layerSlider.slider("disable");
+                }
+                if (self.layerCommandSlider !== undefined) {
+                    self.layerCommandSlider.slider("disable");
+                }
             }
         };
 
-        self.showGCodeViewer = function (response, rstatus) {
-            var par = {
-                target: {
-                    result: response
-                }
-            };
-            GCODE.renderer.clear();
-            GCODE.gCodeReader.loadFile(par);
-
-            if (self.layerSlider !== undefined) {
-                self.layerSlider.slider("disable");
-            }
-            if (self.layerCommandSlider !== undefined) {
-                self.layerCommandSlider.slider("disable");
-            }
+        self._onFileLoaded = function () {
+            self.loadedFilepath = self.cachedPath;
+            self.loadedFileDate = self.cachedDate;
+            self.loadedFileSize = self.cachedSize;
+            self.status = "idle";
+            self.enableReload(true);
         };
 
         self.reload = function () {
             if (!self.enableReload()) return;
-            self.loadFile(self.loadedFilepath, self.loadedFileDate);
+            self.loadFile(
+                self.loadedFilepath,
+                self.loadedFileDate,
+                self.loadedFileSize,
+                true
+            );
         };
 
         self.fromHistoryData = function (data) {
@@ -528,6 +590,7 @@ $(function () {
 
                 self.loadedFilepath = undefined;
                 self.loadedFileDate = undefined;
+                self.loadedFileSize = undefined;
                 self.selectedFile.path(undefined);
                 self.selectedFile.date(undefined);
                 self.selectedFile.size(undefined);
@@ -543,7 +606,8 @@ $(function () {
             if (
                 self.loadedFilepath &&
                 self.loadedFilepath === data.job.file.path &&
-                self.loadedFileDate === data.job.file.date
+                self.loadedFileDate === data.job.file.date &&
+                self.loadedFileSize === data.job.file.size
             ) {
                 if (
                     OctoPrint.coreui.browserTabVisible &&
@@ -554,16 +618,16 @@ $(function () {
                 ) {
                     self._renderPercentage(data.progress.completion);
                 }
-                self.errorCount = 0;
             } else {
-                self.clear();
+                if (self.status === "idle") self.clear();
                 if (
                     data.job.file.path &&
                     data.job.file.origin !== "sdcard" &&
                     self.status !== "request" &&
                     (!self.waitForApproval() ||
                         self.selectedFile.path() !== data.job.file.path ||
-                        self.selectedFile.date() !== data.job.file.date)
+                        self.selectedFile.date() !== data.job.file.date ||
+                        self.selectedFile.size() !== data.job.file.size)
                 ) {
                     self.selectedFile.path(data.job.file.path);
                     self.selectedFile.date(data.job.file.date);
@@ -579,10 +643,15 @@ $(function () {
                         self.waitForApproval(true);
                         self.loadedFilepath = undefined;
                         self.loadedFileDate = undefined;
+                        self.loadedFileSize = undefined;
                     } else {
                         self.waitForApproval(false);
                         if (self.tabActive) {
-                            self.loadFile(data.job.file.path, data.job.file.date);
+                            self.loadFile(
+                                data.job.file.path,
+                                data.job.file.date,
+                                data.job.file.size
+                            );
                         } else {
                             self.needsLoad = true;
                         }
@@ -599,12 +668,17 @@ $(function () {
 
         self.approveLargeFile = function () {
             self.waitForApproval(false);
-            self.loadFile(self.selectedFile.path(), self.selectedFile.date());
+            self.loadFile(
+                self.selectedFile.path(),
+                self.selectedFile.date(),
+                self.selectedFile.size()
+            );
         };
 
         self._onProgress = function (type, percentage) {
             self.ui_progress_type(type);
-            self.ui_progress_percentage(percentage);
+            self.ui_progress_percentage(percentage === undefined ? 100 : percentage);
+            self.ui_progress_busy(percentage === undefined);
         };
 
         self._onModelLoaded = function (model) {
@@ -823,7 +897,11 @@ $(function () {
         self.onTabChange = function (current, previous) {
             self.tabActive = current === "#gcode";
             if (self.tabActive && self.needsLoad) {
-                self.loadFile(self.selectedFile.path(), self.selectedFile.date());
+                self.loadFile(
+                    self.selectedFile.path(),
+                    self.selectedFile.date(),
+                    self.selectedFile.size()
+                );
             }
         };
 
@@ -866,21 +944,25 @@ $(function () {
                 showMoves: self.renderer_showMoves(),
                 showRetracts: self.renderer_showRetracts(),
                 showPrinthead: self.renderer_showPrinthead(),
+                showSegmentStarts: self.renderer_showSegmentStarts(),
+                showDebugArcs: self.renderer_showDebugArcs(),
+                chromeArcFix: self.renderer_chromeArcFix(),
                 showPrevious: self.renderer_showPrevious(),
                 showCurrent: self.renderer_showCurrent(),
                 showNext: self.renderer_showNext(),
                 showFullsize: self.renderer_showFullSize(),
                 showBoundingBox: self.renderer_showBoundingBox(),
-                hideEmptyLayers: self.reader_hideEmptyLayers(),
-                sortLayers: self.reader_sortLayers()
+                showLayerBoundingBox: self.renderer_showLayerBoundingBox(),
+                hideEmptyLayers: self.reader_hideEmptyLayers()
+                // reader_forceCompression is never saved.
             });
         };
         self._fromLocalStorage = function () {
             self.resetOptions();
 
             var current = loadFromLocalStorage(optionsLocalStorageKey);
-            if (current["centerViewport"] !== undefined)
-                self.renderer_centerViewport(current["centerViewport"]);
+            if (current["centerViewPort"] !== undefined)
+                self.renderer_centerViewport(current["centerViewPort"]);
             if (current["zoomOnModel"] !== undefined)
                 self.renderer_zoomOnModel(current["zoomOnModel"]);
             if (current["showMoves"] !== undefined)
@@ -889,6 +971,12 @@ $(function () {
                 self.renderer_showRetracts(current["showRetracts"]);
             if (current["showPrinthead"] !== undefined)
                 self.renderer_showPrinthead(current["showPrinthead"]);
+            if (current["showSegmentStarts"] !== undefined)
+                self.renderer_showSegmentStarts(current["showSegmentStarts"]);
+            if (current["showDebugArcs"] !== undefined)
+                self.renderer_showDebugArcs(current["showDebugArcs"]);
+            if (current["chromeArcFix"] !== undefined)
+                self.renderer_chromeArcFix(current["chromeArcFix"]);
             if (current["showPrevious"] !== undefined)
                 self.renderer_showPrevious(current["showPrevious"]);
             if (current["showCurrent"] !== undefined)
@@ -899,10 +987,10 @@ $(function () {
                 self.renderer_showFullSize(current["showFullsize"]);
             if (current["showBoundingBox"] !== undefined)
                 self.renderer_showBoundingBox(current["showBoundingBox"]);
+            if (current["showLayerBoundingBox"] !== undefined)
+                self.renderer_showLayerBoundingBox(current["showLayerBoundingBox"]);
             if (current["hideEmptyLayers"] !== undefined)
                 self.reader_hideEmptyLayers(current["hideEmptyLayers"]);
-            if (current["sortLayers"] !== undefined)
-                self.reader_sortLayers(current["sortLayers"]);
         };
     }
 
