@@ -1,14 +1,9 @@
-# -*- coding: utf-8 -*-
-from __future__ import absolute_import, division, print_function, unicode_literals
-
 __license__ = "GNU Affero General Public License http://www.gnu.org/licenses/agpl.html"
 __copyright__ = "Copyright (C) 2018 The OctoPrint Project - Released under terms of the AGPLv3 License"
 
-import io
 import os
 
-import yaml
-from flask import jsonify, make_response, request, url_for
+from flask import abort, jsonify, request, url_for
 from flask_babel import gettext
 from werkzeug.exceptions import BadRequest
 from werkzeug.utils import secure_filename
@@ -19,11 +14,7 @@ from octoprint.access.permissions import Permissions
 from octoprint.server import NO_CONTENT
 from octoprint.server.util.flask import no_firstrun_access, redirect_to_tornado
 from octoprint.settings import settings
-
-try:
-    from os import scandir
-except ImportError:
-    from scandir import scandir
+from octoprint.util import is_hidden_path, yaml
 
 
 class LoggingPlugin(
@@ -32,7 +23,6 @@ class LoggingPlugin(
     octoprint.plugin.TemplatePlugin,
     octoprint.plugin.BlueprintPlugin,
 ):
-
     # Additional permissions hook
 
     def get_additional_permissions(self):
@@ -88,8 +78,12 @@ class LoggingPlugin(
     @Permissions.PLUGIN_LOGGING_MANAGE.require(403)
     def delete_log(self, filename):
         secure = os.path.join(settings().getBaseFolder("logs"), secure_filename(filename))
-        if not os.path.exists(secure):
-            return make_response("File not found: %s" % filename, 404)
+        if (
+            not os.path.exists(secure)
+            or is_hidden_path(secure)
+            or not filename.endswith(".log")
+        ):
+            abort(404)
 
         os.remove(secure)
 
@@ -114,15 +108,15 @@ class LoggingPlugin(
     @Permissions.PLUGIN_LOGGING_MANAGE.require(403)
     def set_logging_levels_api(self):
         if "application/json" not in request.headers["Content-Type"]:
-            return make_response("Expected content-type JSON", 400)
+            abort(400, description="Expected content-type JSON")
 
         try:
             json_data = request.json
         except BadRequest:
-            return make_response("Malformed JSON body in request", 400)
+            abort(400, description="Malformed JSON body in request")
 
         if not isinstance(json_data, dict):
-            return make_response("Invalid log level configuration", 400)
+            abort(400, description="Invalid log level configuration")
 
         # TODO validate further
 
@@ -132,6 +126,9 @@ class LoggingPlugin(
     def is_blueprint_protected(self):
         return False
 
+    def is_blueprint_csrf_protected(self):
+        return True
+
     def _get_usage(self):
         import psutil
 
@@ -141,7 +138,13 @@ class LoggingPlugin(
     def _getLogFiles(self):
         files = []
         basedir = settings().getBaseFolder("logs", check_writable=False)
-        for entry in scandir(basedir):
+        for entry in os.scandir(basedir):
+            if is_hidden_path(entry.path) or not entry.name.endswith(".log"):
+                continue
+
+            if not entry.is_file():
+                continue
+
             files.append(
                 {
                     "name": entry.name,
@@ -177,10 +180,7 @@ class LoggingPlugin(
 
         config_from_file = {}
         if os.path.exists(logging_file) and os.path.isfile(logging_file):
-            import yaml
-
-            with io.open(logging_file, "rt", encoding="utf-8") as f:
-                config_from_file = yaml.safe_load(f)
+            config_from_file = yaml.load_from_file(path=logging_file)
         return config_from_file
 
     def _get_logging_levels(self):
@@ -188,11 +188,11 @@ class LoggingPlugin(
         if config is None or not isinstance(config, dict):
             return {}
 
-        return dict(
-            (key, value.get("level"))
+        return {
+            key: value.get("level")
             for key, value in config.get("loggers", {}).items()
             if isinstance(value, dict) and "level" in value
-        )
+        }
 
     def _set_logging_levels(self, new_levels):
         import logging
@@ -230,15 +230,13 @@ class LoggingPlugin(
         with octoprint.util.atomic_write(
             self._get_logging_file(), mode="wt", max_permissions=0o666
         ) as f:
-            yaml.safe_dump(
-                config, f, default_flow_style=False, indent=2, allow_unicode=True
-            )
+            yaml.save_to_file(config, file=f, pretty=True)
 
         # set runtime logging levels now
         for logger, level in new_levels.items():
             level = logging.getLevelName(level)
 
-            self._logger.info("Setting logger {} level to {}".format(logger, level))
+            self._logger.info(f"Setting logger {logger} level to {level}")
             self._logger.manager.loggerDict[logger].setLevel(level)
 
     def _is_managed_logger(self, logger):
@@ -277,7 +275,7 @@ __plugin_disabling_discouraged__ = gettext(
     "the web interface."
 )
 __plugin_license__ = "AGPLv3"
-__plugin_pythoncompat__ = ">=2.7,<4"
+__plugin_pythoncompat__ = ">=3.7,<4"
 __plugin_implementation__ = LoggingPlugin()
 __plugin_hooks__ = {
     "octoprint.access.permissions": __plugin_implementation__.get_additional_permissions
