@@ -1,6 +1,3 @@
-# -*- coding: utf-8 -*-
-from __future__ import absolute_import, division, print_function, unicode_literals
-
 """
 This module holds the standard implementation of the :class:`PrinterInterface` and it helpers.
 """
@@ -15,13 +12,7 @@ import os
 import threading
 import time
 
-try:
-    from immutabledict import immutabledict
-except ImportError:
-    # Python 2
-    from frozendict import frozendict as immutabledict
-
-from past.builtins import basestring, long
+from frozendict import frozendict
 
 import octoprint.util.json
 from octoprint import util as util
@@ -40,7 +31,7 @@ from octoprint.settings import settings
 from octoprint.util import InvariantContainer
 from octoprint.util import comm as comm
 from octoprint.util import get_fully_qualified_classname as fqcn
-from octoprint.util import monotonic_time, to_unicode
+from octoprint.util import to_unicode
 
 
 class Printer(PrinterInterface, comm.MachineComPrintCallback):
@@ -53,10 +44,10 @@ class Printer(PrinterInterface, comm.MachineComPrintCallback):
         from collections import deque
 
         self._logger = logging.getLogger(__name__)
-        self._logger_job = logging.getLogger("{}.job".format(__name__))
+        self._logger_job = logging.getLogger(f"{__name__}.job")
 
         self._dict = (
-            immutabledict
+            frozendict
             if settings().getBoolean(["devel", "useFrozenDictForPrinterState"])
             else dict
         )
@@ -65,24 +56,15 @@ class Printer(PrinterInterface, comm.MachineComPrintCallback):
         self._fileManager = fileManager
         self._printerProfileManager = printerProfileManager
 
-        # state
-        # TODO do we really need to hold the temperature here?
-        self._temp = None
-        self._bedTemp = None
-        self._chamberTemp = None
-        self._targetTemp = None
-        self._targetBedTemp = None
-        self._targetChamberTemp = None
-        self._temps = TemperatureHistory(
+        self._temps = DataHistory(
+            cutoff=settings().getInt(["temperature", "cutoff"]) * 60
+        )
+        self._markings = DataHistory(
             cutoff=settings().getInt(["temperature", "cutoff"]) * 60
         )
-        self._tempBacklog = []
 
         self._messages = deque([], 300)
-        self._messageBacklog = []
-
         self._log = deque([], 300)
-        self._logBacklog = []
 
         self._state = None
 
@@ -96,7 +78,6 @@ class Printer(PrinterInterface, comm.MachineComPrintCallback):
         # sd handling
         self._sdPrinting = False
         self._sdStreaming = False
-        self._sdFilelistAvailable = threading.Event()
         self._streamingFinishedCallback = None
         self._streamingFailedCallback = None
 
@@ -113,13 +94,11 @@ class Printer(PrinterInterface, comm.MachineComPrintCallback):
             try:
                 estimator = hook()
                 if estimator is not None:
-                    self._logger.info(
-                        "Using print time estimator provided by {}".format(name)
-                    )
+                    self._logger.info(f"Using print time estimator provided by {name}")
                     self._estimator_factory = estimator
             except Exception:
                 self._logger.exception(
-                    "Error while processing analysis queues from {}".format(name),
+                    f"Error while processing analysis queues from {name}",
                     extra={"plugin": name},
                 )
 
@@ -183,6 +162,7 @@ class Printer(PrinterInterface, comm.MachineComPrintCallback):
         eventManager().subscribe(
             Events.METADATA_STATISTICS_UPDATED, self._on_event_MetadataStatisticsUpdated
         )
+        eventManager().subscribe(Events.CHART_MARKED, self._on_event_ChartMarked)
 
         self._handle_connect_hooks = plugin_manager().get_hooks(
             "octoprint.printer.handle_connect"
@@ -203,7 +183,7 @@ class Printer(PrinterInterface, comm.MachineComPrintCallback):
 
     @property
     def firmware_info(self):
-        return immutabledict(self._firmware_info) if self._firmware_info else None
+        return frozendict(self._firmware_info) if self._firmware_info else None
 
     # ~~ handling of PrinterCallbacks
 
@@ -286,11 +266,11 @@ class Printer(PrinterInterface, comm.MachineComPrintCallback):
             try:
                 additional = hook(initial=initial)
                 if additional and isinstance(additional, dict):
-                    octoprint.util.json.dump({name: additional})
+                    octoprint.util.json.dumps({name: additional})
                     plugin_data[name] = additional
             except ValueError:
                 self._logger.exception(
-                    "Invalid additional data from plugin {}".format(name),
+                    f"Invalid additional data from plugin {name}",
                     extra={"plugin": name},
                 )
             except Exception:
@@ -326,6 +306,17 @@ class Printer(PrinterInterface, comm.MachineComPrintCallback):
                     self._selectedFile["user"],
                 )
 
+    # ~~ chart marking insertions
+
+    def _on_event_ChartMarked(self, event, data):
+        self._markings.append(
+            {
+                "type": data.get("type", "unknown"),
+                "label": data.get("label"),
+                "time": data.get("time", time.time()),
+            }
+        )
+
     # ~~ progress plugin reporting
 
     def _reportPrintProgressToPlugins(self, progress):
@@ -369,15 +360,13 @@ class Printer(PrinterInterface, comm.MachineComPrintCallback):
         for name, hook in self._handle_connect_hooks.items():
             try:
                 if hook(
-                    self, port=port, baudrate=baudrate, profile=profile, *args, **kwargs
+                    self, *args, port=port, baudrate=baudrate, profile=profile, **kwargs
                 ):
-                    self._logger.info(
-                        "Connect signalled as handled by plugin {}".format(name)
-                    )
+                    self._logger.info(f"Connect signalled as handled by plugin {name}")
                     return
             except Exception:
                 self._logger.exception(
-                    "Exception while handling connect in plugin {}".format(name),
+                    f"Exception while handling connect in plugin {name}",
                     extra={"plugin": name},
                 )
 
@@ -414,7 +403,6 @@ class Printer(PrinterInterface, comm.MachineComPrintCallback):
         self._firmware_info = None
 
     def get_transport(self, *args, **kwargs):
-
         if self._comm is None:
             return None
 
@@ -472,7 +460,7 @@ class Printer(PrinterInterface, comm.MachineComPrintCallback):
         # this code preserves existing CamelCase
         event_name = name[0].upper() + name[1:]
 
-        event_start = "GcodeScript{}Running".format(event_name)
+        event_start = f"GcodeScript{event_name}Running"
         payload = context.get("event", None) if isinstance(context, dict) else None
 
         eventManager().fire(event_start, payload)
@@ -486,21 +474,19 @@ class Printer(PrinterInterface, comm.MachineComPrintCallback):
         if not result and must_be_set:
             raise UnknownScript(name)
 
-        event_end = "GcodeScript{}Finished".format(event_name)
+        event_end = f"GcodeScript{event_name}Finished"
         eventManager().fire(event_end, payload)
 
     def jog(self, axes, relative=True, speed=None, *args, **kwargs):
-        if isinstance(axes, basestring):
+        if isinstance(axes, str):
             # legacy parameter format, there should be an amount as first anonymous positional arguments too
             axis = axes
 
             if not len(args) >= 1:
                 raise ValueError("amount not set")
             amount = args[0]
-            if not isinstance(amount, (int, long, float)):
-                raise ValueError(
-                    "amount must be a valid number: {amount}".format(amount=amount)
-                )
+            if not isinstance(amount, (int, float)):
+                raise ValueError(f"amount must be a valid number: {amount}")
 
             axes = {}
             axes[axis] = amount
@@ -517,15 +503,15 @@ class Printer(PrinterInterface, comm.MachineComPrintCallback):
                 )
 
         command = "G0 {}".format(
-            " ".join(["{}{}".format(axis.upper(), amt) for axis, amt in axes.items()])
+            " ".join([f"{axis.upper()}{amt}" for axis, amt in axes.items()])
         )
 
         if speed is None:
             printer_profile = self._printerProfileManager.get_current_or_default()
-            speed = min([printer_profile["axes"][axis]["speed"] for axis in axes])
+            speed = min(printer_profile["axes"][axis]["speed"] for axis in axes)
 
         if speed and not isinstance(speed, bool):
-            command += " F{}".format(speed)
+            command += f" F{speed}"
 
         if relative:
             commands = ["G91", command, "G90"]
@@ -536,12 +522,10 @@ class Printer(PrinterInterface, comm.MachineComPrintCallback):
 
     def home(self, axes, *args, **kwargs):
         if not isinstance(axes, (list, tuple)):
-            if isinstance(axes, basestring):
+            if isinstance(axes, str):
                 axes = [axes]
             else:
-                raise ValueError(
-                    "axes is neither a list nor a string: {axes}".format(axes=axes)
-                )
+                raise ValueError(f"axes is neither a list nor a string: {axes}")
 
         validated_axes = list(
             filter(
@@ -549,7 +533,7 @@ class Printer(PrinterInterface, comm.MachineComPrintCallback):
             )
         )
         if len(axes) != len(validated_axes):
-            raise ValueError("axes contains invalid axes: {axes}".format(axes=axes))
+            raise ValueError(f"axes contains invalid axes: {axes}")
 
         self.commands(
             [
@@ -561,10 +545,8 @@ class Printer(PrinterInterface, comm.MachineComPrintCallback):
         )
 
     def extrude(self, amount, speed=None, *args, **kwargs):
-        if not isinstance(amount, (int, long, float)):
-            raise ValueError(
-                "amount must be a valid number: {amount}".format(amount=amount)
-            )
+        if not isinstance(amount, (int, float)):
+            raise ValueError(f"amount must be a valid number: {amount}")
 
         printer_profile = self._printerProfileManager.get_current_or_default()
 
@@ -585,7 +567,7 @@ class Printer(PrinterInterface, comm.MachineComPrintCallback):
 
     def change_tool(self, tool, *args, **kwargs):
         if not PrinterInterface.valid_tool_regex.match(tool):
-            raise ValueError('tool must match "tool[0-9]+": {tool}'.format(tool=tool))
+            raise ValueError(f'tool must match "tool[0-9]+": {tool}')
 
         tool_num = int(tool[len("tool") :])
         self.commands(
@@ -601,10 +583,8 @@ class Printer(PrinterInterface, comm.MachineComPrintCallback):
                 )
             )
 
-        if not isinstance(value, (int, long, float)) or value < 0:
-            raise ValueError(
-                "value must be a valid number >= 0: {value}".format(value=value)
-            )
+        if not isinstance(value, (int, float)) or value < 0:
+            raise ValueError(f"value must be a valid number >= 0: {value}")
 
         tags = kwargs.get("tags", set()) | {"trigger:printer.set_temperature"}
 
@@ -614,15 +594,15 @@ class Printer(PrinterInterface, comm.MachineComPrintCallback):
             shared_nozzle = printer_profile["extruder"]["sharedNozzle"]
             if extruder_count > 1 and not shared_nozzle:
                 toolNum = int(heater[len("tool") :])
-                self.commands("M104 T{} S{}".format(toolNum, value), tags=tags)
+                self.commands(f"M104 T{toolNum} S{value}", tags=tags)
             else:
-                self.commands("M104 S{}".format(value), tags=tags)
+                self.commands(f"M104 S{value}", tags=tags)
 
         elif heater == "bed":
-            self.commands("M140 S{}".format(value), tags=tags)
+            self.commands(f"M140 S{value}", tags=tags)
 
         elif heater == "chamber":
-            self.commands("M141 S{}".format(value), tags=tags)
+            self.commands(f"M141 S{value}", tags=tags)
 
     def set_temperature_offset(self, offsets=None, *args, **kwargs):
         if offsets is None:
@@ -635,17 +615,13 @@ class Printer(PrinterInterface, comm.MachineComPrintCallback):
             filter(lambda x: PrinterInterface.valid_heater_regex.match(x), offsets.keys())
         )
         validated_values = list(
-            filter(lambda x: isinstance(x, (int, long, float)), offsets.values())
+            filter(lambda x: isinstance(x, (int, float)), offsets.values())
         )
 
         if len(validated_keys) != len(offsets):
-            raise ValueError(
-                "offsets contains invalid keys: {offsets}".format(offsets=offsets)
-            )
+            raise ValueError(f"offsets contains invalid keys: {offsets}")
         if len(validated_values) != len(offsets):
-            raise ValueError(
-                "offsets contains invalid values: {offsets}".format(offsets=offsets)
-            )
+            raise ValueError(f"offsets contains invalid values: {offsets}")
 
         if self._comm is None:
             return
@@ -654,16 +630,16 @@ class Printer(PrinterInterface, comm.MachineComPrintCallback):
         self._setOffsets(self._comm.getOffsets())
 
     def _convert_rate_value(self, factor, min_val=None, max_val=None):
-        if not isinstance(factor, (int, float, long)):
+        if not isinstance(factor, (int, float)):
             raise ValueError("factor is not a number")
 
         if isinstance(factor, float):
             factor = int(factor * 100)
 
         if min_val and factor < min_val:
-            raise ValueError("factor must be a value >={}".format(min_val))
+            raise ValueError(f"factor must be a value >={min_val}")
         elif max_val and factor > max_val:
-            raise ValueError("factor must be a value <={}".format(max_val))
+            raise ValueError(f"factor must be a value <={max_val}")
 
         return factor
 
@@ -747,6 +723,9 @@ class Printer(PrinterInterface, comm.MachineComPrintCallback):
 
         return self._comm.getFilePosition()
 
+    def get_markings(self):
+        return self._markings
+
     def start_print(self, pos=None, user=None, *args, **kwargs):
         """
         Starts the currently loaded print job.
@@ -844,11 +823,11 @@ class Printer(PrinterInterface, comm.MachineComPrintCallback):
             return self._comm.getErrorString()
 
     def get_current_data(self, *args, **kwargs):
-        return util.thaw_immutabledict(self._stateMonitor.get_current_data())
+        return util.thaw_frozendict(self._stateMonitor.get_current_data())
 
     def get_current_job(self, *args, **kwargs):
         currentData = self._stateMonitor.get_current_data()
-        return util.thaw_immutabledict(currentData["job"])
+        return util.thaw_frozendict(currentData["job"])
 
     def get_current_temperatures(self, *args, **kwargs):
         if self._comm is not None:
@@ -856,37 +835,22 @@ class Printer(PrinterInterface, comm.MachineComPrintCallback):
         else:
             offsets = {}
 
-        result = {}
-        if self._temp is not None:
-            for tool in self._temp.keys():
-                result["tool%d" % tool] = {
-                    "actual": self._temp[tool][0],
-                    "target": self._temp[tool][1],
-                    "offset": offsets[tool]
-                    if tool in offsets and offsets[tool] is not None
-                    else 0,
-                }
-        if self._bedTemp is not None:
-            result["bed"] = {
-                "actual": self._bedTemp[0],
-                "target": self._bedTemp[1],
-                "offset": offsets["bed"]
-                if "bed" in offsets and offsets["bed"] is not None
-                else 0,
-            }
-        if self._chamberTemp is not None:
-            result["chamber"] = {
-                "actual": self._chamberTemp[0],
-                "target": self._chamberTemp[1],
-                "offset": offsets["chamber"]
-                if "chamber" in offsets and offsets["chamber"] is not None
-                else 0,
-            }
+        last = self._temps.last
+        if last is None:
+            return {}
 
-        return result
+        return {
+            key: {
+                "actual": value["actual"],
+                "target": value["target"],
+                "offset": offsets[key] if offsets.get(key) is not None else 0,
+            }
+            for key, value in last.items()
+            if key != "time"
+        }
 
     def get_temperature_history(self, *args, **kwargs):
-        return self._temps
+        return list(self._temps)
 
     def get_current_connection(self, *args, **kwargs):
         if self._comm is None:
@@ -926,7 +890,8 @@ class Printer(PrinterInterface, comm.MachineComPrintCallback):
     def is_ready(self, *args, **kwargs):
         return (
             self.is_operational()
-            and not self.is_printing()
+            and not self._comm.isBusy()
+            # isBusy is true when paused
             and not self._comm.isStreaming()
         )
 
@@ -947,7 +912,7 @@ class Printer(PrinterInterface, comm.MachineComPrintCallback):
 
         return list(
             map(
-                lambda x: {"name": x[0][1:], "size": x[1], "display": x[2]},
+                lambda x: {"name": x[0][1:], "size": x[1], "display": x[2], "date": x[3]},
                 self._comm.getSdFiles(),
             )
         )
@@ -1003,7 +968,7 @@ class Printer(PrinterInterface, comm.MachineComPrintCallback):
                     sd_upload_succeeded,
                     sd_upload_failed,
                     *args,
-                    **kwargs
+                    **kwargs,
                 )
                 if result is not None:
                     return result
@@ -1040,12 +1005,10 @@ class Printer(PrinterInterface, comm.MachineComPrintCallback):
         if tags is None:
             tags = set()
 
-        remote_name = self._get_free_remote_name(filename)
         self._create_estimator("stream")
-        self._comm.startFileTransfer(
+        remote_name = self._comm.startFileTransfer(
             path,
             filename,
-            "/" + remote_name,
             special=not valid_file_type(filename, "gcode"),
             tags=tags | {"trigger:printer.add_sd_file"},
         )
@@ -1082,12 +1045,11 @@ class Printer(PrinterInterface, comm.MachineComPrintCallback):
         """
         if not self._comm or not self._comm.isSdReady():
             return
-        self._sdFilelistAvailable.clear()
         self._comm.refreshSdFiles(
-            tags=kwargs.get("tags", set()) | {"trigger:printer.refresh_sd_files"}
+            tags=kwargs.get("tags", set()) | {"trigger:printer.refresh_sd_files"},
+            blocking=blocking,
+            timeout=kwargs.get("timeout", 10),
         )
-        if blocking and not self.is_printing():
-            self._sdFilelistAvailable.wait(kwargs.get("timeout", 10))
 
     # ~~ state monitoring
 
@@ -1195,7 +1157,7 @@ class Printer(PrinterInterface, comm.MachineComPrintCallback):
                         printTimeLeft = int(printTimeLeft)
                 except Exception:
                     self._logger.exception(
-                        "Error while estimating print time via {}".format(estimator)
+                        f"Error while estimating print time via {estimator}"
                     )
 
         return self._dict(
@@ -1235,18 +1197,11 @@ class Printer(PrinterInterface, comm.MachineComPrintCallback):
 
         self._temps.append(data)
 
-        self._temp = tools
-        self._bedTemp = bed
-        self._chamberTemp = chamber
-        self._custom = custom
-
         self._stateMonitor.add_temperature(self._dict(**data))
 
     def _validateJob(self, filename, sd):
         if not valid_file_type(filename, type="machinecode"):
-            raise InvalidFileType(
-                "{} is not a machinecode file, cannot print".format(filename)
-            )
+            raise InvalidFileType(f"{filename} is not a machinecode file, cannot print")
 
         if sd:
             return
@@ -1265,7 +1220,7 @@ class Printer(PrinterInterface, comm.MachineComPrintCallback):
                 )
             )
 
-    def _setJobData(self, filename, filesize, sd, user=None):
+    def _setJobData(self, filename, filesize, sd, user=None, data=None):
         with self._selectedFileMutex:
             if filename is not None:
                 if sd:
@@ -1318,11 +1273,11 @@ class Printer(PrinterInterface, comm.MachineComPrintCallback):
             date = None
             filament = None
             display_name = name_in_storage
+
             if path_on_disk:
-                # Use a string for mtime because it could be float and the
+                # Use an int for mtime because it could be float and the
                 # javascript needs to exact match
-                if not sd:
-                    date = int(os.stat(path_on_disk).st_mtime)
+                date = int(os.stat(path_on_disk).st_mtime)
 
                 try:
                     fileData = self._fileManager.get_metadata(
@@ -1369,6 +1324,10 @@ class Printer(PrinterInterface, comm.MachineComPrintCallback):
                         self._selectedFile["estimatedPrintTime"] = estimatedPrintTime
                         self._selectedFile["estimatedPrintTimeType"] = "analysis"
 
+            elif data:
+                display_name = data.longname
+                date = data.timestamp
+
             self._stateMonitor.set_job_data(
                 self._dict(
                     file=self._dict(
@@ -1414,6 +1373,7 @@ class Printer(PrinterInterface, comm.MachineComPrintCallback):
                 temps=list(self._temps),
                 logs=list(self._log),
                 messages=list(self._messages),
+                markings=list(self._markings),
             )
 
             plugin_data = self._get_additional_plugin_data(initial=False)
@@ -1530,12 +1490,17 @@ class Printer(PrinterInterface, comm.MachineComPrintCallback):
             if self._comm is not None:
                 self._comm = None
 
+            with self._selectedFileMutex:
+                if self._selectedFile is not None:
+                    eventManager().fire(Events.FILE_DESELECTED)
+                self._setJobData(None, None, None)
+
             self._updateProgressData()
             self._setCurrentZ(None)
-            self._setJobData(None, None, None)
             self._setOffsets(None)
             self._addTemperatureData()
             self._printerProfileManager.deselect()
+
             eventManager().fire(Events.DISCONNECTED)
 
         self._setState(state, state_string=state_string, error_string=error_string)
@@ -1578,9 +1543,8 @@ class Printer(PrinterInterface, comm.MachineComPrintCallback):
 
     def on_comm_sd_files(self, files):
         eventManager().fire(Events.UPDATED_FILES, {"type": "printables"})
-        self._sdFilelistAvailable.set()
 
-    def on_comm_file_selected(self, full_path, size, sd, user=None):
+    def on_comm_file_selected(self, full_path, size, sd, user=None, data=None):
         if full_path is not None:
             payload = self._payload_for_print_job_event(
                 location=FileDestinations.SDCARD if sd else FileDestinations.LOCAL,
@@ -1603,7 +1567,7 @@ class Printer(PrinterInterface, comm.MachineComPrintCallback):
                 "Print job deselected - user: {}".format(user if user else "n/a")
             )
 
-        self._setJobData(full_path, size, sd, user=user)
+        self._setJobData(full_path, size, sd, user=user, data=data)
         self._stateMonitor.set_state(
             self._dict(
                 text=self.get_state_string(),
@@ -1626,6 +1590,10 @@ class Printer(PrinterInterface, comm.MachineComPrintCallback):
         payload = self._payload_for_print_job_event(print_job_user=user, action_user=user)
         if payload:
             eventManager().fire(Events.PRINT_STARTED, payload)
+            eventManager().fire(
+                Events.CHART_MARKED,
+                {"type": "print", "label": "Start"},
+            )
             self._logger_job.info(
                 "Print job started - origin: {}, path: {}, owner: {}, user: {}".format(
                     payload.get("origin"),
@@ -1649,6 +1617,10 @@ class Printer(PrinterInterface, comm.MachineComPrintCallback):
         payload = self._payload_for_print_job_event()
         if payload:
             payload["time"] = self._comm.getPrintTime()
+            eventManager().fire(
+                Events.CHART_MARKED,
+                {"type": "done", "label": "Done"},
+            )
             self._updateProgressData(
                 completion=1.0,
                 filepos=payload["size"],
@@ -1725,6 +1697,10 @@ class Printer(PrinterInterface, comm.MachineComPrintCallback):
             payload["time"] = self._comm.getPrintTime()
 
             eventManager().fire(Events.PRINT_CANCELLED, payload)
+            eventManager().fire(
+                Events.CHART_MARKED,
+                {"type": "cancel", "label": "Cancel"},
+            )
             self._logger_job.info(
                 "Print job cancelled - origin: {}, path: {}, owner: {}, user: {}".format(
                     payload.get("origin"),
@@ -1776,6 +1752,10 @@ class Printer(PrinterInterface, comm.MachineComPrintCallback):
                     payload.get("user"),
                 )
             )
+            eventManager().fire(
+                Events.CHART_MARKED,
+                {"type": "pause", "label": "Pause"},
+            )
             if not suppress_script:
                 self.script(
                     "afterPrintPaused",
@@ -1788,6 +1768,10 @@ class Printer(PrinterInterface, comm.MachineComPrintCallback):
         payload = self._payload_for_print_job_event(action_user=user)
         if payload:
             eventManager().fire(Events.PRINT_RESUMED, payload)
+            eventManager().fire(
+                Events.CHART_MARKED,
+                {"type": "resume", "label": "Resume"},
+            )
             self._logger_job.info(
                 "Print job resumed - origin: {}, path: {}, owner: {}, user: {}".format(
                     payload.get("origin"),
@@ -1932,7 +1916,7 @@ class Printer(PrinterInterface, comm.MachineComPrintCallback):
         return result
 
 
-class StateMonitor(object):
+class StateMonitor:
     def __init__(
         self,
         interval=0.5,
@@ -1966,7 +1950,7 @@ class StateMonitor(object):
         self._progress_lock = threading.Lock()
         self._resends_lock = threading.Lock()
 
-        self._last_update = monotonic_time()
+        self._last_update = time.monotonic()
         self._worker = threading.Thread(target=self._work)
         self._worker.daemon = True
         self._worker.start()
@@ -2052,7 +2036,7 @@ class StateMonitor(object):
             while True:
                 self._change_event.wait()
 
-                now = monotonic_time()
+                now = time.monotonic()
                 delta = now - self._last_update
                 additional_wait_time = self._interval - delta
                 if additional_wait_time > 0:
@@ -2061,7 +2045,7 @@ class StateMonitor(object):
                 with self._state_lock:
                     data = self.get_current_data()
                     self._update_callback(data)
-                    self._last_update = monotonic_time()
+                    self._last_update = time.monotonic()
                     self._change_event.clear()
         except Exception:
             logging.getLogger(__name__).exception(
@@ -2091,11 +2075,22 @@ class StateMonitor(object):
         }
 
 
-class TemperatureHistory(InvariantContainer):
+class DataHistory(InvariantContainer):
     def __init__(self, cutoff=30 * 60):
-        def temperature_invariant(data):
+        def data_invariant(data):
             data.sort(key=lambda x: x["time"])
             now = int(time.time())
             return [item for item in data if item["time"] >= now - cutoff]
 
-        InvariantContainer.__init__(self, guarantee_invariant=temperature_invariant)
+        InvariantContainer.__init__(self, guarantee_invariant=data_invariant)
+        self._last = None
+
+    @property
+    def last(self):
+        return self._last
+
+    def append(self, item):
+        try:
+            return super().append(item)
+        finally:
+            self._last = self._data[-1] if len(self._data) else None
