@@ -1,13 +1,9 @@
-# -*- coding: utf-8 -*-
-from __future__ import absolute_import, division, print_function, unicode_literals
-
 __author__ = "Marc Hannappel <salandora@gmail.com>"
 __license__ = "GNU Affero General Public License http://www.gnu.org/licenses/agpl.html"
 __copyright__ = "Copyright (C) 2017 The OctoPrint Project - Released under terms of the AGPLv3 License"
 
-from flask import abort, jsonify, make_response, request
+from flask import abort, jsonify, request
 from flask_login import current_user
-from werkzeug.exceptions import BadRequest
 
 import octoprint.access.groups as groups
 import octoprint.access.users as users
@@ -29,29 +25,23 @@ def get_permissions():
 
 @api.route("/access/groups", methods=["GET"])
 @no_firstrun_access
-@Permissions.SETTINGS.require(403)
+@Permissions.ADMIN.require(403)
 def get_groups():
     return jsonify(groups=list(map(lambda g: g.as_dict(), groupManager.groups)))
 
 
 @api.route("/access/groups", methods=["POST"])
 @no_firstrun_access
-@Permissions.SETTINGS.require(403)
+@Permissions.ADMIN.require(403)
 def add_group():
-    if "application/json" not in request.headers["Content-Type"]:
-        return make_response("Expected content-type JSON", 400)
-
-    try:
-        data = request.get_json()
-    except BadRequest:
-        return make_response("Malformed JSON body in request", 400)
+    data = request.get_json()
 
     if "key" not in data:
-        return make_response("Missing mandatory key field", 400)
+        abort(400, description="key is missing")
     if "name" not in data:
-        return make_response("Missing mandatory name field", 400)
+        abort(400, description="name is missing")
     if "permissions" not in data:
-        return make_response("Missing mandatory permissions field", 400)
+        abort(400, description="permissions are missing")
 
     key = data["key"]
     name = data["name"]
@@ -76,7 +66,7 @@ def add_group():
 
 @api.route("/access/groups/<key>", methods=["GET"])
 @no_firstrun_access
-@Permissions.SETTINGS.require(403)
+@Permissions.ADMIN.require(403)
 def get_group(key):
     group = groupManager.find_group(key)
     if group is not None:
@@ -87,15 +77,9 @@ def get_group(key):
 
 @api.route("/access/groups/<key>", methods=["PUT"])
 @no_firstrun_access
-@Permissions.SETTINGS.require(403)
+@Permissions.ADMIN.require(403)
 def update_group(key):
-    if "application/json" not in request.headers["Content-Type"]:
-        return make_response("Expected content-type JSON", 400)
-
-    try:
-        data = request.get_json()
-    except BadRequest:
-        return make_response("Malformed JSON body in request", 400)
+    data = request.get_json()
 
     try:
         kwargs = {}
@@ -123,7 +107,7 @@ def update_group(key):
 
 @api.route("/access/groups/<key>", methods=["DELETE"])
 @no_firstrun_access
-@Permissions.SETTINGS.require(403)
+@Permissions.ADMIN.require(403)
 def remove_group(key):
     try:
         groupManager.remove_group(key)
@@ -139,32 +123,23 @@ def remove_group(key):
 
 @api.route("/access/users", methods=["GET"])
 @no_firstrun_access
-@Permissions.SETTINGS.require(403)
+@Permissions.ADMIN.require(403)
 def get_users():
     return jsonify(users=list(map(lambda u: u.as_dict(), userManager.get_all_users())))
 
 
 @api.route("/access/users", methods=["POST"])
 @no_firstrun_access
-@Permissions.SETTINGS.require(403)
+@Permissions.ADMIN.require(403)
 def add_user():
-    if "application/json" not in request.headers["Content-Type"]:
-        return make_response("Expected content-type JSON", 400)
-
-    try:
-        data = request.get_json()
-    except BadRequest:
-        return make_response("Malformed JSON body in request", 400)
-
-    if data is None:
-        return make_response("Malformed JSON body in request", 400)
+    data = request.get_json()
 
     if "name" not in data:
-        return make_response("Missing mandatory name field", 400)
+        abort(400, description="name is missing")
     if "password" not in data:
-        return make_response("Missing mandatory password field", 400)
+        abort(400, description="password is missing")
     if "active" not in data:
-        return make_response("Missing mandatory active field", 400)
+        abort(400, description="active is missing")
 
     name = data["name"]
     password = data["password"]
@@ -177,6 +152,8 @@ def add_user():
         userManager.add_user(name, password, active, permissions, groups)
     except users.UserAlreadyExists:
         abort(409)
+    except users.InvalidUsername:
+        abort(400, "Username invalid")
     return get_users()
 
 
@@ -202,17 +179,11 @@ def get_user(username):
 
 @api.route("/access/users/<username>", methods=["PUT"])
 @no_firstrun_access
-@Permissions.SETTINGS.require(403)
+@Permissions.ADMIN.require(403)
 def update_user(username):
     user = userManager.find_user(username)
     if user is not None:
-        if "application/json" not in request.headers["Content-Type"]:
-            return make_response("Expected content-type JSON", 400)
-
-        try:
-            data = request.get_json()
-        except BadRequest:
-            return make_response("Malformed JSON body in request", 400)
+        data = request.get_json()
 
         # change groups
         if "groups" in data:
@@ -224,9 +195,6 @@ def update_user(username):
             permissions = data["permissions"]
             userManager.change_user_permissions(username, permissions)
 
-        if data is None:
-            return make_response("Malformed JSON body in request", 400)
-
         # change activation
         if "active" in data:
             userManager.change_user_activation(
@@ -240,7 +208,7 @@ def update_user(username):
 
 @api.route("/access/users/<username>", methods=["DELETE"])
 @no_firstrun_access
-@Permissions.SETTINGS.require(403)
+@Permissions.ADMIN.require(403)
 def remove_user(username):
     try:
         userManager.remove_user(username)
@@ -258,30 +226,31 @@ def change_password_for_user(username):
     if (
         current_user is not None
         and not current_user.is_anonymous
-        and (current_user.get_name() == username or current_user.is_admin)
+        and (
+            current_user.get_name() == username
+            or current_user.has_permission(Permissions.ADMIN)
+        )
     ):
-        if "application/json" not in request.headers["Content-Type"]:
-            return make_response("Expected content-type JSON", 400)
-
-        try:
-            data = request.get_json()
-        except BadRequest:
-            return make_response("Malformed JSON body in request", 400)
-
-        if data is None:
-            return make_response("Malformed JSON body in request", 400)
+        data = request.get_json()
 
         if "password" not in data or not data["password"]:
-            return make_response("password is missing from request", 400)
+            abort(400, description="new password is missing")
+
+        if not current_user.has_permission(Permissions.ADMIN) or "current" in data:
+            if "current" not in data or not data["current"]:
+                abort(400, description="current password is missing")
+
+            if not userManager.check_password(username, data["current"]):
+                abort(403, description="Invalid current password")
 
         try:
             userManager.change_user_password(username, data["password"])
         except users.UnknownUser:
-            return make_response(("Unknown user: %s" % username, 404, []))
+            abort(404)
 
         return jsonify(SUCCESS)
     else:
-        return make_response(("Forbidden", 403, []))
+        abort(403)
 
 
 @api.route("/access/users/<username>/settings", methods=["GET"])
@@ -295,12 +264,12 @@ def get_settings_for_user(username):
             and not current_user.has_permission(Permissions.ADMIN)
         )
     ):
-        return make_response("Forbidden", 403)
+        abort(403)
 
     try:
         return jsonify(userManager.get_all_user_settings(username))
     except users.UnknownUser:
-        return make_response("Unknown user: %s" % username, 404)
+        abort(404)
 
 
 @api.route("/access/users/<username>/settings", methods=["PATCH"])
@@ -314,21 +283,15 @@ def change_settings_for_user(username):
             and not current_user.has_permission(Permissions.ADMIN)
         )
     ):
-        return make_response("Forbidden", 403)
+        abort(403)
 
-    try:
-        data = request.get_json()
-    except BadRequest:
-        return make_response("Malformed JSON body in request", 400)
-
-    if data is None:
-        return make_response("Malformed JSON body in request", 400)
+    data = request.get_json()
 
     try:
         userManager.change_user_settings(username, data)
         return jsonify(SUCCESS)
     except users.UnknownUser:
-        return make_response("Unknown user: %s" % username, 404)
+        abort(404)
 
 
 @api.route("/access/users/<username>/apikey", methods=["DELETE"])
@@ -345,10 +308,10 @@ def delete_apikey_for_user(username):
         try:
             userManager.delete_api_key(username)
         except users.UnknownUser:
-            return make_response(("Unknown user: %s" % username, 404, []))
+            abort(404)
         return jsonify(SUCCESS)
     else:
-        return make_response(("Forbidden", 403, []))
+        abort(403)
 
 
 @api.route("/access/users/<username>/apikey", methods=["POST"])
@@ -368,10 +331,10 @@ def generate_apikey_for_user(username):
         try:
             apikey = userManager.generate_api_key(username)
         except users.UnknownUser:
-            return make_response(("Unknown user: %s" % username, 404, []))
+            abort(404)
         return jsonify({"apikey": apikey})
     else:
-        return make_response(("Forbidden", 403, []))
+        abort(403)
 
 
 def _to_external_permissions(*permissions):
