@@ -1,6 +1,3 @@
-# -*- coding: utf-8 -*-
-from __future__ import absolute_import, division, print_function, unicode_literals
-
 __author__ = "Gina Häußge <osd@foosel.net>"
 __license__ = "GNU Affero General Public License http://www.gnu.org/licenses/agpl.html"
 __copyright__ = "Copyright (C) 2014 The OctoPrint Project - Released under terms of the AGPLv3 License"
@@ -8,7 +5,6 @@ __copyright__ = "Copyright (C) 2014 The OctoPrint Project - Released under terms
 import re
 
 from flask import Response, abort, jsonify, request
-from past.builtins import basestring, long, unicode
 
 from octoprint.access.permissions import Permissions
 from octoprint.printer import UnknownScript
@@ -94,7 +90,7 @@ def printerToolCommand():
     ##~~ tool selection
     if command == "select":
         tool = data["tool"]
-        if not isinstance(tool, basestring) or re.match(validation_regex, tool) is None:
+        if not isinstance(tool, str) or re.match(validation_regex, tool) is None:
             abort(400, description="tool is invalid")
 
         printer.change_tool(tool, tags=tags)
@@ -108,7 +104,7 @@ def printerToolCommand():
         for tool, value in targets.items():
             if re.match(validation_regex, tool) is None:
                 abort(400, description="targets contains invalid tool")
-            if not isinstance(value, (int, long, float)):
+            if not isinstance(value, (int, float)):
                 abort(400, description="targets contains invalid value")
             validated_values[tool] = value
 
@@ -125,7 +121,7 @@ def printerToolCommand():
         for tool, value in offsets.items():
             if re.match(validation_regex, tool) is None:
                 abort(400, description="offsets contains invalid tool")
-            if not isinstance(value, (int, long, float)) or not -50 <= value <= 50:
+            if not isinstance(value, (int, float)) or not -50 <= value <= 50:
                 abort(400, description="offsets contains invalid value")
             validated_values[tool] = value
 
@@ -140,13 +136,13 @@ def printerToolCommand():
 
         amount = data["amount"]
         speed = data.get("speed", None)
-        if not isinstance(amount, (int, long, float)):
+        if not isinstance(amount, (int, float)):
             abort(400, description="amount is invalid")
         printer.extrude(amount, speed=speed, tags=tags)
 
     elif command == "flowrate":
         factor = data["factor"]
-        if not isinstance(factor, (int, long, float)):
+        if not isinstance(factor, (int, float)):
             abort(400, description="factor is invalid")
         try:
             printer.flow_rate(factor, tags=tags)
@@ -191,7 +187,7 @@ def printerBedCommand():
         target = data["target"]
 
         # make sure the target is a number
-        if not isinstance(target, (int, long, float)):
+        if not isinstance(target, (int, float)):
             abort(400, description="target is invalid")
 
         # perform the actual temperature command
@@ -202,7 +198,7 @@ def printerBedCommand():
         offset = data["offset"]
 
         # make sure the offset is valid
-        if not isinstance(offset, (int, long, float)) or not -50 <= offset <= 50:
+        if not isinstance(offset, (int, float)) or not -50 <= offset <= 50:
             abort(400, description="offset is invalid")
 
         # set the offsets
@@ -253,7 +249,7 @@ def printerChamberCommand():
         target = data["target"]
 
         # make sure the target is a number
-        if not isinstance(target, (int, long, float)):
+        if not isinstance(target, (int, float)):
             abort(400, description="target is invalid")
 
         # perform the actual temperature command
@@ -264,7 +260,7 @@ def printerChamberCommand():
         offset = data["offset"]
 
         # make sure the offset is valid
-        if not isinstance(offset, (int, long, float)) or not -50 <= offset <= 50:
+        if not isinstance(offset, (int, float)) or not -50 <= offset <= 50:
             abort(400, description="offset is invalid")
 
         # set the offsets
@@ -316,7 +312,7 @@ def printerPrintheadCommand():
         for axis in valid_axes:
             if axis in data:
                 value = data[axis]
-                if not isinstance(value, (int, long, float)):
+                if not isinstance(value, (int, float)):
                     abort(400, description="axis value is invalid")
                 validated_values[axis] = value
 
@@ -340,7 +336,7 @@ def printerPrintheadCommand():
 
     elif command == "feedrate":
         factor = data["factor"]
-        if not isinstance(factor, (int, long, float)):
+        if not isinstance(factor, (int, float)):
             abort(400, description="factor is invalid")
         try:
             printer.feed_rate(factor, tags=tags)
@@ -400,14 +396,8 @@ def printerCommand():
     if not printer.is_operational():
         abort(409, description="Printer is not operational")
 
-    if "application/json" not in request.headers["Content-Type"]:
-        abort(400, description="Expected content type JSON")
-
     data = request.get_json()
 
-    if data is None:
-        abort(400, description="Malformed JSON body in request")
-
     if "command" in data and "commands" in data:
         abort(400, description="'command' and 'commands' are mutually exclusive")
     elif ("command" in data or "commands" in data) and "script" in data:
@@ -467,13 +457,12 @@ def _get_temperature_data(preprocessor):
     tempData = printer.get_current_temperatures()
 
     if "history" in request.values and request.values["history"] in valid_boolean_trues:
-        tempHistory = printer.get_temperature_history()
+        history = printer.get_temperature_history()
 
         limit = 300
-        if "limit" in request.values and unicode(request.values["limit"]).isnumeric():
+        if "limit" in request.values and str(request.values["limit"]).isnumeric():
             limit = int(request.values["limit"])
 
-        history = list(tempHistory)
         limit = min(limit, len(history))
 
         tempData.update(
