@@ -56,9 +56,10 @@ function ItemListHelper(
         for (var i = 0; i < itemList.length; i++) {
             if (matcher(itemList[i])) {
                 self.selectedItem(itemList[i]);
-                break;
+                return true;
             }
         }
+        return false;
     };
 
     self.selectNone = function () {
@@ -618,7 +619,8 @@ function formatDate(unixTimestamp, options) {
         options = {seconds: false};
     }
 
-    if (!unixTimestamp) return "-";
+    var placeholder = options.placeholder || "-";
+    if (!unixTimestamp) return placeholder;
 
     var format = gettext(/* L10N: Date format */ "YYYY-MM-DD HH:mm");
     if (options.seconds) {
@@ -628,8 +630,9 @@ function formatDate(unixTimestamp, options) {
     return moment.unix(unixTimestamp).format(format);
 }
 
-function formatTimeAgo(unixTimestamp) {
-    if (!unixTimestamp) return "-";
+function formatTimeAgo(unixTimestamp, placeholder) {
+    placeholder = placeholder || "-";
+    if (!unixTimestamp) return placeholder;
     return moment.unix(unixTimestamp).fromNow();
 }
 
@@ -651,13 +654,25 @@ function cleanTemperature(temp, offThreshold) {
     return temp;
 }
 
-function formatTemperature(temp, showF, offThreshold) {
+function formatTemperature(temp, showF, offThreshold, returnUnicode) {
     if (temp === undefined || !_.isNumber(temp)) return "-";
     if (offThreshold !== undefined && temp < offThreshold) return gettext("off");
+
+    var degreeSymbol = "&deg;";
+    if (returnUnicode !== undefined && returnUnicode) {
+        degreeSymbol = "\u00B0";
+    }
+
     if (showF) {
-        return _.sprintf("%.1f&deg;C (%.1f&deg;F)", temp, (temp * 9) / 5 + 32);
+        return _.sprintf(
+            "%.1f%sC (%.1f%sF)",
+            temp,
+            degreeSymbol,
+            (temp * 9) / 5 + 32,
+            degreeSymbol
+        );
     } else {
-        return _.sprintf("%.1f&deg;C", temp);
+        return _.sprintf("%.1f%sC", temp, degreeSymbol);
     }
 }
 
@@ -1484,7 +1499,7 @@ var sizeObservable = function (observable) {
 var getQueryParameterByName = function (name, url) {
     // from http://stackoverflow.com/a/901144/2028598
     if (!url) {
-        url = window.location.href;
+        url = fetchWindowLocation().href;
     }
     name = name.replace(/[\[\]]/g, "\\$&");
     var regex = new RegExp("[?&]" + name + "(=([^&#]*)|&|#|$)"),
@@ -1541,37 +1556,75 @@ var copyToClipboard = function (text) {
     temp.remove();
 };
 
-var determineWebcamStreamType = function (streamUrl) {
-    if (streamUrl) {
-        var lastDotPosition = streamUrl.lastIndexOf(".");
-        var firstQuotationSignPosition = streamUrl.indexOf("?");
-        if (
-            lastDotPosition != -1 &&
-            firstQuotationSignPosition != -1 &&
-            lastDotPosition >= firstQuotationSignPosition
-        ) {
-            throw "Malformed URL. Cannot determine stream type.";
-        }
+var getExternalHostUrl = function () {
+    var loc = fetchWindowLocation();
+    var port = "";
+    if (
+        (loc.protocol === "http:" && loc.port !== "80") ||
+        (loc.protocol === "https:" && loc.port !== "443")
+    ) {
+        port = ":" + loc.port;
+    }
+    return loc.protocol + "//" + loc.hostname + port;
+};
 
-        // If we have found a dot, try to extract the extension.
-        if (lastDotPosition > -1) {
-            if (firstQuotationSignPosition > -1) {
-                var extension = streamUrl.slice(
-                    lastDotPosition + 1,
-                    firstQuotationSignPosition - 1
-                );
-            } else {
-                var extension = streamUrl.slice(lastDotPosition + 1);
-            }
-            if (extension.toLowerCase() == "m3u8") {
-                return "hls";
-            }
-        }
-        // By default, 'mjpg' is the stream type.
-        return "mjpg";
+var validateWebcamUrl = function (streamUrl) {
+    if (!streamUrl) {
+        return false;
+    }
+
+    var lower = streamUrl.toLowerCase();
+    var toParse = streamUrl;
+
+    if (lower.startsWith("//")) {
+        // protocol relative
+        toParse = fetchWindowLocation().protocol + streamUrl;
+    } else if (lower.startsWith("/")) {
+        // host relative
+        toParse = getExternalHostUrl() + streamUrl;
+    } else if (
+        lower.startsWith("http:") ||
+        lower.startsWith("https:") ||
+        lower.startsWith("webrtc:") ||
+        lower.startsWith("webrtcs:")
+    ) {
+        // absolute & supported protocol
+        toParse = streamUrl;
     } else {
+        return false;
+    }
+
+    try {
+        return new URL(toParse);
+    } catch (e) {
+        return false;
+    }
+};
+
+var determineWebcamStreamType = function (streamUrl) {
+    if (!streamUrl) {
         throw "Empty streamUrl. Cannot determine stream type.";
     }
+
+    var parsed = validateWebcamUrl(streamUrl);
+    if (!parsed) {
+        throw "Invalid streamUrl. Cannot determine stream type.";
+    }
+
+    if (parsed.protocol === "webrtc:" || parsed.protocol === "webrtcs:") {
+        return "webrtc";
+    }
+
+    var lastDotPosition = parsed.pathname.lastIndexOf(".");
+    if (lastDotPosition !== -1) {
+        var extension = parsed.pathname.substring(lastDotPosition + 1);
+        if (extension.toLowerCase() === "m3u8") {
+            return "hls";
+        }
+    }
+
+    // By default, 'mjpg' is the stream type.
+    return "mjpg";
 };
 
 var saveToLocalStorage = function (key, data) {
@@ -1579,18 +1632,20 @@ var saveToLocalStorage = function (key, data) {
     localStorage[key] = JSON.stringify(data);
 };
 
-var loadFromLocalStorage = function (key) {
-    if (!Modernizr.localstorage) return {};
+var loadFromLocalStorage = function (key, defaultValue) {
+    defaultValue = defaultValue === undefined ? {} : defaultValue;
+
+    if (!Modernizr.localstorage) return defaultValue;
 
     var currentString = localStorage[key];
     var current;
     if (currentString === undefined) {
-        current = {};
+        current = defaultValue;
     } else {
         try {
             current = JSON.parse(currentString);
         } catch (ex) {
-            current = {};
+            current = defaultValue;
         }
     }
     return current;
@@ -1619,3 +1674,86 @@ var deepMerge = function (target, source) {
 
     return target;
 };
+
+var isWebRTCAvailable = function () {
+    return (
+        typeof RTCPeerConnection == "function" &&
+        typeof RTCPeerConnection.prototype.addEventListener == "function" &&
+        typeof RTCPeerConnection.prototype.addTransceiver == "function" &&
+        typeof RTCPeerConnection.prototype.createOffer == "function" &&
+        typeof RTCPeerConnection.prototype.setLocalDescription == "function" &&
+        typeof RTCPeerConnection.prototype.removeEventListener == "function" &&
+        typeof RTCPeerConnection.prototype.addEventListener == "function" &&
+        typeof RTCPeerConnection.prototype.setRemoteDescription == "function"
+    );
+};
+
+var negotiateWebRTC = function (streamUrl) {
+    pc.addTransceiver("video", {direction: "recvonly"});
+    pc.addTransceiver("audio", {direction: "recvonly"});
+    return pc
+        .createOffer()
+        .then(function (offer) {
+            return pc.setLocalDescription(offer);
+        })
+        .then(function () {
+            // Wait for ICE gathering to complete
+            return new Promise(function (resolve) {
+                if (pc.iceGatheringState === "complete") {
+                    resolve();
+                } else {
+                    function checkState() {
+                        if (pc.iceGatheringState === "complete") {
+                            pc.removeEventListener("icegatheringstatechange", checkState);
+                            resolve();
+                        }
+                    }
+                    pc.addEventListener("icegatheringstatechange", checkState);
+                }
+            });
+        })
+        .then(function () {
+            var offer = pc.localDescription;
+            // webrtc://host.com becomes http://host.com
+            // webrtcs://host.com becomes https://host.com
+            streamUrl = "http" + streamUrl.slice("webrtc".length);
+            return $.ajax({
+                url: streamUrl,
+                type: "POST",
+                dataType: "json",
+                data: JSON.stringify({
+                    sdp: offer.sdp,
+                    type: offer.type
+                }),
+                contentType: "application/json; charset=UTF-8"
+            });
+        })
+        .then(function (response) {
+            return pc.setRemoteDescription(response);
+        })
+        .catch(function (e) {
+            console.error(e);
+        });
+};
+
+var startWebRTC = function (videoElement, streamUrl, iceServers) {
+    var config = {
+        sdpSemantics: "unified-plan"
+    };
+    if (iceServers) {
+        config.iceServers = [{urls: iceServers}];
+    }
+    pc = new RTCPeerConnection(config);
+    pc.addEventListener("track", function (evt) {
+        if (evt.track.kind == "video") {
+            videoElement.srcObject = evt.streams[0];
+        }
+    });
+
+    negotiateWebRTC(streamUrl);
+    return pc;
+};
+
+var fetchWindowLocation = function () {
+    return window.location;
+};
