@@ -6,7 +6,13 @@ $(function () {
         self.settings = parameters[1];
         self.access = parameters[2];
 
+        self.allViewModels = undefined;
+
         self.stateString = ko.observable(undefined);
+        self.errorString = ko.observable(undefined);
+        self.hasErrorString = ko.pureComputed(function () {
+            return !!self.errorString();
+        });
 
         self.resendCount = ko.observable(0);
         self.resendTotalTransmitted = ko.observable(0);
@@ -42,7 +48,7 @@ $(function () {
         self.enablePrint = ko.pureComputed(function () {
             return (
                 self.isOperational() &&
-                self.isReady() &&
+                (self.isReady() || self.isPaused()) &&
                 !self.isPrinting() &&
                 !self.isCancelling() &&
                 !self.isPausing() &&
@@ -243,10 +249,6 @@ $(function () {
 
         self.userString = ko.pureComputed(function () {
             var user = self.user();
-            if (!CONFIG_ACCESS_CONTROL || user === "_dummy") {
-                return "";
-            }
-
             if (user === "_api") {
                 user = "API client";
             }
@@ -258,7 +260,7 @@ $(function () {
         self.dateString = ko.pureComputed(function () {
             var date = self.filedate();
             if (!date) {
-                return "";
+                return gettext("unknown");
             }
 
             return formatDate(date, {seconds: true});
@@ -291,6 +293,7 @@ $(function () {
             var prevPaused = self.isPaused();
 
             self.stateString(gettext(data.text));
+            self.errorString(data.error);
             self.isErrorOrClosed(data.flags.closedOrError);
             self.isOperational(data.flags.operational);
             self.isPaused(data.flags.paused);
@@ -432,8 +435,27 @@ $(function () {
                     }
                 });
             } else {
+                var proceed = function (p) {
+                    var prevented = false;
+                    var callback = function () {
+                        OctoPrint.job.start();
+                    };
+
+                    callViewModels(
+                        self.allViewModels,
+                        "onBeforePrintStart",
+                        function (method) {
+                            prevented = prevented || method(callback) === false;
+                        }
+                    );
+
+                    if (!prevented) {
+                        callback();
+                    }
+                };
+
                 if (!self.settings.feature_printStartConfirmation()) {
-                    OctoPrint.job.start();
+                    proceed();
                 } else {
                     showConfirmationDialog({
                         message: gettext(
@@ -442,9 +464,7 @@ $(function () {
                         question: gettext("Do you want to start the print job now?"),
                         cancel: gettext("No"),
                         proceed: gettext("Yes"),
-                        onproceed: function () {
-                            OctoPrint.job.start();
-                        },
+                        onproceed: proceed,
                         nofade: true
                     });
                 }
@@ -478,6 +498,10 @@ $(function () {
                 });
             }
         };
+
+        self.onAllBound = function (allViewModels) {
+            self.allViewModels = allViewModels;
+        };
     }
 
     OCTOPRINT_VIEWMODELS.push({
